#
# Chum::Analysis::ApplicationI
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Analysis::ApplicationI - An application interface

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut


package Chum::Analysis::ApplicationI;

use vars qw(@ISA @APPLICATION_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;

@ISA = qw( Bio::Root::Root );
BEGIN {
        @APPLICATION_PARAMS = qw();
        foreach my $attr (@APPLICATION_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
        my($class, @args) = @_;
        my $self = $class->SUPER::new(@args);
        while (@args) {
                my $attr = shift @args;
                my $value = shift @args;
                $self->{$attr} = $value;
        }
        return $self;
}

sub AUTOLOAD {
        my $self = shift;
        my $attr = $AUTOLOAD;
        $attr =~ s/.*:://;

        $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
        $self->{$attr} = shift if @_;
        return $self->{$attr};
}

sub convert_output_to_gff {
	shift->throw_not_implemented();
}

sub convert_output_to_xml {
	shift->throw_not_implemented();	
}

sub program {
	my $self = shift;
	if(@_) {
		my $program = shift;
		$self->{'program'} = $program;
	}
	return $self->{'program'};   
}

sub run_variation_analysis {
	shift->throw_not_implemented();
}

1;