#
# Chum::Analysis::ExtData::NhgriDNaseISite
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Analysis::ExtData::NhgriDNaseISite - Find NHGRI DNaseI Hypersensitive sites from external data downloaded from UCSC
(Regulatory Potential Metric)

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut


package Chum::Analysis::ExtData::NhgriDNaseISite;

use vars qw(@ISA @NDHS_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;
use Data::Dumper;

@ISA = qw( Bio::Root::Root );
BEGIN {
        @NDHS_PARAMS = qw(ext_file);
        foreach my $attr (@NDHS_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
        my($class, @args) = @_;
        my $self = $class->SUPER::new(@args);
        while (@args) {
                my $attr = shift @args;
                my $value = shift @args;
                $self->{$attr} = $value;
        }
		if (!defined($self->{'ext_file'})) {
			$self->{'ext_file'} = $ENV{'CHUM_INSTALL_DIR'} . "/external_data/nhgriDnaseHs.txt";
		}
		unless(-e $self->{'ext_file'}) { $self->throw("External DNaseI File could not be found"); }
		
		$self->_load();

        return $self;
}

sub AUTOLOAD {
        my $self = shift;
        my $attr = $AUTOLOAD;
        $attr =~ s/.*:://;

        $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
        $self->{$attr} = shift if @_;
        return $self->{$attr};
}

sub _load() {
	my $self = shift;
	open(DNHS, $self->{'ext_file'}) || $self->throw("Cannot open external data file: " . $self->{'ext_file'});
	my %data_array;
	while (my $line = <DNHS>) {
		chomp($line);
		if ($line =~ /[0-9]+[\s]+chr([\S]+)[\s]+([0-9]+)[\s]+([0-9]+)[\s][\S]+[\s]+([0-9]+)/gi) {
			my $chr = $1;
			my $start = $2;
			my $end = $3;
			my $score = $4;
			if (defined($data_array{$chr})) {
				my $arrayref = $data_array{$chr};
				push @$arrayref, {'start' => $start, 'end' => $end, 'score' => $score};
				$data_array{$chr} = $arrayref;
			} else {
				my @array = ();
				push @array, {'start' => $start, 'end' => $end, 'score' => $score};
				$data_array{$chr} = \@array;
			}
		} else {
			$self->throw("Unknown line in external data: $line");
		}
	}
	close(DNHS);
	$self->{'data'} = \%data_array;
	print STDERR "NHGRI DNaseI Hypersensitive Site File Loaded\n";
}

sub in_dnase1_hs_site() {
	my $self = shift;
	my $chromosome = shift;
	my $start = shift;
	my $end = shift;

	if ($start > $end) {
		my $atmp = $end;
		$end = $start;
		$start = $atmp;
	}

	if (!defined($self->{'data'}->{$chromosome})) {
		$self->throw("No data found for chromosome $chromosome");
	} else {
		my $arrayref = $self->{'data'}->{$chromosome};
		foreach my $value (@$arrayref) {
			if ($end >= $value->{'start'} && $start <= $value->{'end'}) {
				return $value->{'score'};
			}
		}
	}
	return 0;
}

1;