package Chum::DB::MatrixDB;

use vars qw(@ISA @MATRIXDB_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;
use Data::Dumper;

@ISA = qw( Bio::Root::Root );

BEGIN {
	@MATRIXDB_PARAMS = qw(db);
	foreach my $attr (@MATRIXDB_PARAMS)
		{ $OK_FIELD{$attr}++; }
}

sub new {
	my($class, @args) = @_;
	my $self = $class->SUPER::new(@args);
	while (@args) {
		my $attr = shift @args;
		my $value = shift @args;
		$self->$attr($value);
	}
	return $self;
}

sub AUTOLOAD {
    my $self = shift;
    my $attr = $AUTOLOAD;
    $attr =~ s/.*:://;

    $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
    $self->{$attr} = shift if @_;
    return $self->{$attr};
}

sub matrixset {
	my $self = shift;
	if(@_) {
		my $matrix_set = shift;
		if (!$matrix_set->isa("TFBS::MatrixSet")) {
			$self->throw("ERROR: Matrix set is not a TFBS::MatrixSet object"); 
		} else {
			$self->{'_matrix_set'} = $matrix_set;	
		}
	}
	return $self->{'_matrix_set'};
}

sub _find_all_matrixset {
	my $self = shift;
	return $self->{'db'}->get_all_Matrices('PWM');
}

sub _find_pwm_matrixset {
	my $self = shift;
	my $factor_names_arrayref = shift;
	if (!defined($factor_names_arrayref)) { $self->throw("Need an arrayref of factors");}
	my $matrixset = TFBS::MatrixSet->new();
	foreach my $factor (@$factor_names_arrayref) {
		eval {
			my $pwm = $self->_find_pwm_matrix($factor);
			$matrixset->add_Matrix($pwm);
		};
	}
	return $matrixset;
}

sub _find_pwm_matrix {
        my $self = shift;
	my $matrix_name = shift;
        $self->{'matrix'} = $self->{'db'}->get_Matrix_by_name($matrix_name, 'PWM');
	if ($self->{'matrix'}->isa("TFBS::Matrix::PFM")) {
                $self->{'matrix'} = $self->{'matrix'}->to_PWM();
        }
        return $self->{'matrix'};
}

sub _detect {
        my $self = shift;
        my $sequence = shift;
	my $threshold = "80%";
	if(@_) {
		$threshold = shift;	
	}
	if (!defined($self->{'_matrix_set'})) {
		$self->throw("ERROR: matrix_set must be loaded");	
	} elsif (!defined($sequence)) {
		$self->throw("ERROR: No sequence has been inputted as arg1 for detection");	
	}
        my $siteset = $self->{'_matrix_set'}->search_seq(-seqstring => $sequence,
							 -threshold => $threshold); 
        return $siteset;
}

