package Chum::DB::Oreganno;

use vars qw(@ISA @OREGANNO_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;
use SOAP::Lite;

@ISA = qw( Bio::Root::Root );

BEGIN {
	@OREGANNO_PARAMS = qw();
	foreach my $attr (@OREGANNO_PARAMS)
		{ $OK_FIELD{$attr}++; }
}

sub new {
	my($class, @args) = @_;
	my $self = $class->SUPER::new(@args);
	while (@args) {
		my $attr = shift @args;
		my $value = shift @args;
		$self->$attr($value);
    }
    return $self;
}

sub AUTOLOAD {
    my $self = shift;
    my $attr = $AUTOLOAD;
    $attr =~ s/.*:://;

    $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
    $self->{$attr} = shift if @_;
    return $self->{$attr};
}

sub _get_connection {
	my $self = shift;
	return SOAP::Lite->uri('OregannoServerImpl')->proxy('http://www.oreganno.org/oregano/soap/', timeout => 100000);
}

sub get_record_from_id {
	my $self = shift;
	my $id = shift;

	my $osa = $self->_get_connection;
	my $response = $osa->searchRecords(	SOAP::Data->name(field => SOAP::Data->type(string => "stable_id")),
						SOAP::Data->name(query => SOAP::Data->type(string => "$id")));
        if ($response->fault) {
            $self->throw($response->faultstring);
        } else {
        my $result_str = $response->result;
    	my @records = split '\|', $result_str;
		if (scalar(@records) != 1) {
			return undef;
		} else {
			my $response2 = $osa->fetchRecord(SOAP::Data->name(record_stable_id => SOAP::Data->type(string => $records[0])));
                        if ($response2->fault) {
                                $self->throw($response2->faultstring);
                        } else {
                                my $record_obj = $response2->result;
                                if (!defined($record_obj)) {
                                        return undef;
                                } else {
                                        return $record_obj;    
                                }
                        }
                        
		}
	}
}

sub get_variations_arrayref_from_regulatory_variant_record {
	my $self = shift;
        my $record = shift;
        if (!defined($record)) {$self->throw("Could not find ORegAnno record for " . $record->{'record'}->{'stableId'}); }
        elsif ($record->{'record'}->{'type'} ne "REGULATORY POLYMORPHISM" && $record->{'record'}->{'type'} ne "REGULATORY HAPLOTYPE") { $self->throw("Entered record is not a regulatory polymorphism or haplotype"); }
        else {
                my @variations = @{$record->{'record'}->{'variationSet'}->{'variation'}};
		return \@variations;
	}
}

sub get_dbsnp_id_from_variation {
	my $self = shift;
	my $variation = shift;
	if (!defined($variation)) { $self->throw("No variation defined"); }
	else {
		if (defined($variation->{'regulatoryVariationCrossReferenceSet'})) {
                	my @rvxrefs = @{$variation->{'regulatoryVariationCrossReferenceSet'}->{'regulatoryVariationCrossReference'}};
                        if (defined(@rvxrefs)) {
                        	foreach my $rvxref (@rvxrefs) {
                                	if ($rvxref->{'source'} eq 'dbSNP') {
                                        	if (defined($rvxref->{'snpId'})) {
                                                	return $rvxref->{'snpId'};
						}
					}
				}
			}
		}
	}
	return undef;
}

1;
