#
# Chum::Result::MotifPrediction::Hit
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Result::MotifPrediction::Hit - A hit from a motif prediction analysis run

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut


package Chum::Result::MotifPrediction::Hit;

use vars qw(@ISA @H_PARAMS %OK_FIELD $AUTOLOAD);

use strict;

@ISA = qw( Bio::Root::Root );
BEGIN {
    @H_PARAMS = qw(name score sequence start strand);
    foreach my $attr (@H_PARAMS)
        { $OK_FIELD{$attr}++; }
}

sub new {
    my($class, @args) = @_;
    my $self = $class->SUPER::new(@args);
    while (@args) {
        my $attr = shift @args;
        my $value = shift @args;
        $self->{$attr} = $value;
    }
    return $self;
}

sub AUTOLOAD {
    my $self = shift;
    my $attr = $AUTOLOAD;
    $attr =~ s/.*:://;

    $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
    $self->{$attr} = shift if @_;
    return $self->{$attr};
}

sub name {
    my $self = shift;
    return $self->{'name'}; 
}

sub score {
    my $self = shift;
    return $self->{'score'};  
}

sub sequence {
    my $self = shift;
    return $self->{'sequence'};   
}

sub start {
    my $self = shift;
    return $self->{'start'};   
}

sub strand {
    my $self = shift;
    return $self->{'strand'};   
}

sub load {
    my $self = shift;
    my $h_node = shift;
    if (!defined($h_node)) { $self->throw("No XML::Twig::Elt object defined for loading"); }
    unless ($h_node->isa("XML::Twig::Elt")) { $self->throw("Loading object is not a XML::Twig::Elt object"); }
    
    $self->{'name'} = $h_node->att('name');
    $self->{'score'} = $h_node->att('score');
    $self->{'sequence'} = $h_node->att('sequence');
    $self->{'start'} = $h_node->att('start');
    $self->{'strand'} = $h_node->att('strand');
}

1;