#
# Chum::Result::MotifPrediction::MatrixIndex
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Result::MotifPrediction::MatrixIndex - An index in a matrix from a motif prediction analysis run

=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut


package Chum::Result::MotifPrediction::MatrixIndex;

use vars qw(@ISA @MI_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;

@ISA = qw( Bio::Root::Root );
BEGIN {
    @MI_PARAMS = qw(A C T G position);
    foreach my $attr (@MI_PARAMS)
        { $OK_FIELD{$attr}++; }
}

sub new {
    my($class, @args) = @_;
    my $self = $class->SUPER::new(@args);
    while (@args) {
        my $attr = shift @args;
        my $value = shift @args;
        $self->{$attr} = $value;
    }
    return $self;
}

sub AUTOLOAD {
    my $self = shift;
    my $attr = $AUTOLOAD;
    $attr =~ s/.*:://;

    $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
    $self->{$attr} = shift if @_;
    return $self->{$attr};
}

sub A {
    my $self = shift;
    return $self->{'A'}; 
}

sub C {
    my $self = shift;
    return $self->{'C'};  
}

sub G {
    my $self = shift;
    return $self->{'G'};   
}

sub T {
    my $self = shift;
    return $self->{'T'};   
}

sub position {
    my $self = shift;
    return $self->{'position'};   
}

sub load {
    my $self = shift;
    my $mi_node = shift;
    if (!defined($mi_node)) { $self->throw("No XML::Twig::Elt object defined for loading"); }
    unless ($mi_node->isa("XML::Twig::Elt")) { $self->throw("Loading object is not a XML::Twig::Elt object"); }
    
    $self->{'A'} = $mi_node->att('A');
    $self->{'C'} = $mi_node->att('C');
    $self->{'G'} = $mi_node->att('G');
    $self->{'T'} = $mi_node->att('T');
    $self->{'position'} = $mi_node->att('position');
}

1;