#
# Chum::Result::MotifPrediction::MotifPredictionXMLReader
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Result::MotifPrediction::MotifPredictionXMLReader - Reads standardized XML results from
motif prediction algorithms output.  This internally standardizes results.

Example:
<output>
    <predicted_motif score="27.51" seq="TTAACTAA">
        <matrix name="All occurrences" type="PFM">
            <index A="213" C="13" G="10" T="16" position="8"/>
            <index A="186" C="38" G="16" T="12" position="7"/>
            <index A="33" C="42" G="4" T="173" position="6"/>
            <index A="32" C="152" G="17" T="51" position="5"/>
            <index A="218" C="7" G="6" T="21" position="4"/>
            <index A="213" C="14" G="4" T="21" position="3"/>
            <index A="13" C="17" G="11" T="211" position="2"/>
            <index A="15" C="17" G="6" T="214" position="1"/>
        </matrix>
        <hit name="-1sequences16" score="95.73" sequence="TTAATTAA" start="135" strand="-1"/>
        <hit name="-1sequences16" score="89.35" sequence="TTAGTGAA" start="330" strand="-1"/>
    </predicted_motif>
    <predicted_motif>
        ...


=head1 SYNOPSIS

=head1 DESCRIPTION

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut


package Chum::Result::MotifPrediction::MotifPredictionXMLReader;

use vars qw(@ISA @MPXMLR_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;
use Chum::Result::MotifPrediction::PredictedMotif;
use XML::Twig;

@ISA = qw( Bio::Root::Root );
BEGIN {
    @MPXMLR_PARAMS = qw();
    foreach my $attr (@MPXMLR_PARAMS)
            { $OK_FIELD{$attr}++; }
}

sub new {
    my($class, @args) = @_;
    my $self = $class->SUPER::new(@args);
    while (@args) {
            my $attr = shift @args;
            my $value = shift @args;
            $self->{$attr} = $value;
    }
    return $self;
}

sub AUTOLOAD {
    my $self = shift;
    my $attr = $AUTOLOAD;
    $attr =~ s/.*:://;

    $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
    $self->{$attr} = shift if @_;
    return $self->{$attr};
}

sub read {
    my $self = shift;
    my $file = shift;
    if (!defined($file)) { $self->throw("No file specified for reading"); }
    unless (-f $file) { $self->throw($file . " does not exist"); }
    my $twig = XML::Twig->new();
    my @predicted_motifs = ();
    eval {
        $twig->parsefile($file);
        my $root = $twig->root;
        foreach my $pm_node ($root->children('predicted_motif')){
            my $predicted_motif = new Chum::Result::MotifPrediction::PredictedMotif();
            $predicted_motif->load($pm_node);
            push @predicted_motifs, $predicted_motif;
        }
    };
    if ($@) {
        $self->throw("Error processing file: " . $@);
    } else {
        return \@predicted_motifs; 
    }
}

1;