#
# Chum::Result::ScoredResult
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Result::ScoredResult - DEPRECATED A scored result from a motif discovery run, includes metadata on the run DEPRECATED

=head1 SYNOPSIS

=head1 DESCRIPTION

Stores information about motif discovery

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut


package Chum::Result::ScoredResult;

use vars qw(@ISA @SCORED_RESULT_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;

@ISA = qw( Bio::Root::Root );
BEGIN {
        @SCORED_RESULT_PARAMS = qw(alleles score species variation_name);
        foreach my $attr (@SCORED_RESULT_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
        my($class, @args) = @_;
        my $self = $class->SUPER::new(@args);
        while (@args) {
                my $attr = shift @args;
                my $value = shift @args;
                $self->{$attr} = $value;
        }
        return $self;
}

sub AUTOLOAD {
        my $self = shift;
        my $attr = $AUTOLOAD;
        $attr =~ s/.*:://;

        $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
        $self->{$attr} = shift if @_;
        return $self->{$attr};
}

sub alleles {
	my $self = shift;
    return $self->{'alleles'};
}

sub score {  
	my $self = shift;
	return $self->{'score'};
}

sub species {
	my $self = shift;
    return $self->{'species'};
}

sub variation_name {
	my $self = shift;
    return $self->{'variation_name'};
}

1;