#
# Chum::Stackanalyzer
# Author: Stephen Montgomery
# Copyright GPL
# You may distribute this module under the same terms as perl itself
# POD documentation - main docs before the code
#

=pod

=head1 NAME

Chum::Stackanalyzer - Runs analysis on a stack

=head1 SYNOPSIS

=head1 DESCRIPTION

Runs the selected analyses on a Stack

=head1 CONTACT

Stephen Montgomery (smontgom@bcsgc.bc.ca)

=head1 METHODS

=cut


package Chum::Stackanalyzer;

use vars qw(@ISA @STACKANALYZER_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;
use Chum::Analysis::Motifsampler;
use Chum::Analysis::Weeder;

@ISA = qw( Bio::Root::Root );
BEGIN {
        @STACKANALYZER_PARAMS = qw(tmpdir);
        foreach my $attr (@STACKANALYZER_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
        my($class, @args) = @_;
        my $self = $class->SUPER::new(@args);
        while (@args) {
                my $attr = shift @args;
                my $value = shift @args;
                $self->{$attr} = $value;
        }

	if ($self->{'tmpdir'}) { $self->throw("tmpdir needs to be specified"); }

        return $self;
}

sub AUTOLOAD {
        my $self = shift;
        my $attr = $AUTOLOAD;
        $attr =~ s/.*:://;

        $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
        $self->{$attr} = shift if @_;
        return $self->{$attr};
}

sub analyze {
	my $self = shift;
	my $stack = shift;
	my $background = shift;
	
	
	

}

####################################################
##Run motifsampler with the default service location
##Return a temporary file containing the results.
####################################################
sub _run_motifsampler {
	my $self = shift;
	my $stack = shift;	
	my $background = shift;

	my $motifsampler = new Chum::Analysis::Motifsampler();

	my $rand = rand(1000000);
	my $background_output_file = $self->{'tmpdir'} . "/background." . $rand . ".txt";
	my $motifsampler_output_file = $self->{'tmpdir'} . "/motifsampler_results." . $rand . ".txt";
	
	my @services = @{$motifsampler->get_motifsampler_background_services};
	my $failed = 1;
	if (defined @services) {
	        if (scalar(@services) > 0) {
	                my $service = @services[0];
			my @parameters = @{$motifsampler->get_input_parameters($service)};
	                $failed = $motifsampler->run_motifsampler_background_service($service,
	                                        $background, \@parameters, $background_output_file);
	       }
	}
	if ($failed) { 
		unlink($background_output_file);
		$self->throw("Could not obtain background"); 
	}
	
	my @mservices = @{$motifsampler->get_motifsampler_services};
	my $mfailed = 1;
	if (defined @mservices) {
        	if (scalar(@mservices) > 0) {
                	my $mservice = @mservices[0];
                	my @mparameters = @{$motifsampler->get_input_parameters($mservice)};
                	$mfailed = $motifsampler->run_motifsampler_service($mservice,
                                        	$stack, $background_output_file,  
						\@mparameters, $motifsampler_output_file);
        	}
	}
	if ($mfailed) { 
		unlink($background_output_file);
		unlink($motifsampler_output_file);
		$self->throw("Could not obtain results");	
	}
	
	return $motifsampler_output_file;
}

####################################################
##Run weeder with the default service location
##Return a temporary file containing the results.
####################################################
sub _run_weeder {
	my $self = shift;
	my $stack = shift;

	my $weeder = new Chum::Analysis::Weeder();

	my $rand = rand(1000000);
	my $weeder_output_file = $self->{'tmpdir'} . "/weeder_results." . $rand . ".txt";
	my $weeder_gff_file = $self->{'tmpdir'} . "/weeder_results." . $rand . ".gff";

	my @wservices = @{$weeder->get_weeder_services};
	my $wstatus;
	if (defined @wservices) {
        	if (scalar(@wservices) > 0) {
                	my $wservice = @wservices[0];
               		my @wparameters = @{$weeder->get_input_parameters($wservice)};
               		$wstatus = $weeder->run_weeder_service($wservice,
                                        	$stack, \@wparameters, $weeder_output_file);
       		}
	}
	if ($wstatus) { die("Could not obtain weeder results"); }

	$weeder->_convert_weeder_output_to_gff($weeder_output_file, $weeder_gff_file);
	unlink($weeder_output_file);
	return $weeder_gff_file;	
}

1;
