package Chum::Utils::ClustalWAligner;

use vars qw(@ISA @CA_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;
use Bio::Tools::Run::Alignment::Clustalw;
use Data::Dumper;

@ISA = qw( Bio::Root::Root );

BEGIN {
	@CA_PARAMS = qw();
	foreach my $attr (@CA_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
	my($class, @args) = @_;
	my $self = $class->SUPER::new(@args);
	while (@args) {
		my $attr = shift @args;
		my $value = shift @args;
		$self->$attr($value);
    }
    return $self;
}

sub AUTOLOAD {
    my $self = shift;
    my $attr = $AUTOLOAD;
    $attr =~ s/.*:://;

    $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
    $self->{$attr} = shift if @_;
    return $self->{$attr};
}

sub run_align {
	my $self = shift;
	my $bio_seq_obj_1 = shift;
	my $bio_seq_obj_2 = shift;
	my @sequence = ($bio_seq_obj_1, $bio_seq_obj_2);
	my $factory = Bio::Tools::Run::Alignment::Clustalw->new();
	$factory->executable("clustalw-1.83");
	my $aln = $factory->align(\@sequence);
	return $aln;
}

sub run_multiple_alignment {
	my $self = shift;
	my $sequence_arrref = shift;
	my @sequence = @$sequence_arrref;
	my $factory = Bio::Tools::Run::Alignment::Clustalw->new();
	$factory->executable("clustalw-1.83");
	my $aln = $factory->align(\@sequence);
	return $aln;
}

sub get_multiple_alignment_tree {
	my $self = shift;
	my $sequence_arrref = shift;
	my @sequence = @$sequence_arrref;
	my $factory = Bio::Tools::Run::Alignment::Clustalw->new();
	$factory->executable("clustalw-1.83");
	my $tree = $factory->tree(\@sequence);
	return $tree;
}

1;
