package Chum::Utils::DistanceCalculator;

use vars qw(@ISA @DC_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;
use Chum::Utils::ClustalWAligner;
use Data::Dumper;

@ISA = qw( Bio::Root::Root );

BEGIN {
    @DC_PARAMS = qw();
    foreach my $attr (@DC_PARAMS)
        { $OK_FIELD{$attr}++; }
}

sub new {
    my($class, @args) = @_;
    my $self = $class->SUPER::new(@args);
    while (@args) {
	my $attr = shift @args;
	my $value = shift @args;
	$self->$attr($value);
    }
    return $self;
}

sub AUTOLOAD {
    my $self = shift;
    my $attr = $AUTOLOAD;
    $attr =~ s/.*:://;

    $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
    $self->{$attr} = shift if @_;
    return $self->{$attr};
}

sub calculate_distance {
    my $self = shift;   
    my $stack_object = shift;
    
    my @sequences = @{$stack_object->orthologous_sequences};
    push @sequences, $stack_object->reference_sequence->sequence; 

    my $clustalw = new Chum::Utils::ClustalWAligner();
    my $tree = $clustalw->get_multiple_alignment_tree(\@sequences);
    my $branch_length = $tree->total_branch_length;
    
    return $branch_length;
}

1;