package Chum::Utils::ExpectancyLoader;

use vars qw(@ISA @EL_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::Root::Root;
use Data::Dumper;

@ISA = qw( Bio::Root::Root );

my @CHROMOSOMES = ('1', '2', '3', '4', '5', '6', '7', '8', '9', '10',
                   '11', '12', '13', '14', '15', '16', '17', '18', '19', '20',
                   '21', '22', 'X', 'Y');
my $INSTALL_DIRECTORY = "/cygdrive/c/documentsandsettings/sm8/mydocuments/PERL/CHuM";
#my $INSTALL_DIRECTORY = $ENV{'CHUM_INSTALL_DIR'}

BEGIN {
	@EL_PARAMS = qw();
	foreach my $attr (@EL_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
	my($class, @args) = @_;
	my $self = $class->SUPER::new(@args);
        while (@args) {
		my $attr = shift @args;
		my $value = shift @args;
		$self->$attr($value);
        }
        return $self;
}

sub AUTOLOAD {
    my $self = shift;
    my $attr = $AUTOLOAD;
    $attr =~ s/.*:://;

    $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
    $self->{$attr} = shift if @_;
    return $self->{$attr};
}

sub get_cpg_expectancies {
	my $self = shift;
        my @cpg_expectancy_files = ();
        foreach my $chr (@CHROMOSOMES) {
                my $cpg_expectancy_file = $INSTALL_DIRECTORY . "/expectancies/cpg." . $chr . ".expect";
                unless(-f $cpg_expectancy_file) {
                        $self->warn("Missing cpg expectancy file: " . $cpg_expectancy_file); 
                } else {
                        push @cpg_expectancy_files, $cpg_expectancy_file;
                }
        }
	return $self->_process_single_score_based_expectancy_file(\@cpg_expectancy_files);
}

sub get_noble_expectancies {
	my $self = shift;
	my @noble_expectancy_files = ();
        foreach my $chr (@CHROMOSOMES) {
                my $noble_expectancy_file = $INSTALL_DIRECTORY . "/expectancies/noble." . $chr . ".expect";
                unless(-f $noble_expectancy_file) {
                        $self->warn("Missing noble expectancy file: " . $noble_expectancy_file); 
                } else {
                        push @noble_expectancy_files, $noble_expectancy_file;
                }
        }
	return $self->_process_single_score_based_expectancy_file(\@noble_expectancy_files);
}

sub get_nhgri_expectancies {
	my $self = shift;
	my @nhgri_expectancy_files = ();
        foreach my $chr (@CHROMOSOMES) {
                my $nhgri_expectancy_file = $INSTALL_DIRECTORY . "/expectancies/nhgri." . $chr . ".expect";
                unless(-f $nhgri_expectancy_file) {
                        $self->warn("Missing nhgri expectancy file: " . $nhgri_expectancy_file); 
                } else {
                        push @nhgri_expectancy_files, $nhgri_expectancy_file;
                }
        }
	return $self->_process_single_score_based_expectancy_file(\@nhgri_expectancy_files);
}

sub get_phastcons_expectancies {
	my $self = shift;
	my @phastcons_expectancy_files = ();
        foreach my $chr (@CHROMOSOMES) {
                my $phastcons_expectancy_file = $INSTALL_DIRECTORY . "/expectancies/phastcons." . $chr . ".expect.2";
                unless(-f $phastcons_expectancy_file) {
                        $self->warn("Missing phastcons expectancy file: " . $phastcons_expectancy_file); 
                } else {
                        push @phastcons_expectancy_files, $phastcons_expectancy_file;
                }
        }
	return $self->_process_single_score_based_expectancy_file(\@phastcons_expectancy_files);
}

sub get_rp_expectancies {
	my $self = shift;
	my @rp_expectancy_files = ();
        foreach my $chr (@CHROMOSOMES) {
                my $rp_expectancy_file = $INSTALL_DIRECTORY . "/expectancies/rp." . $chr . ".expect";
                unless(-f $rp_expectancy_file) {
                        $self->warn("Missing rp expectancy file: " . $rp_expectancy_file); 
                } else {
                        push @rp_expectancy_files, $rp_expectancy_file;
                }
        }
	return $self->_process_single_score_based_expectancy_file(\@rp_expectancy_files);
}

sub get_repeat_expectancies {
	my $self = shift;
	my @repeat_expectancy_files = ();
        foreach my $chr (@CHROMOSOMES) {
                my $repeat_expectancy_file = $INSTALL_DIRECTORY . "/expectancies/repeat." . $chr . ".expect";
                unless(-f $repeat_expectancy_file) {
                        $self->warn("Missing repeat expectancy file: " . $repeat_expectancy_file); 
                } else {
                        push @repeat_expectancy_files, $repeat_expectancy_file;
                }
        }
	return $self->_process_repeat_based_expectancy_file(\@repeat_expectancy_files);
}

##################### PRIVATE FUNCTIONS #################################

sub _process_single_score_based_expectancy_file {
	my $self = shift;
	my $expectancy_files_arrayref = shift;
	my @expectancy_files = @$expectancy_files_arrayref;
	
	my %return_expectancies;
	foreach my $expectancy_file (@expectancy_files) {
		my ($chromosome) = $expectancy_file =~ m/[\S\s]+\.([\S]+)\.expect$/;
		print STDERR "Opening $expectancy_file for chr: $chromosome\n";
		open(RP, $expectancy_file) || $self->throw("Cannot open expectancy file " . $expectancy_file); 
		while (my $line = <RP>) {
			chomp($line);
			if ($line =~ /##TRANSCRIPT[\s]+([\S\s]+)/g) {
				my $index_line = $1;
				my @indices = split('\t', $index_line);
				$return_expectancies{$chromosome}{'indices'} = \@indices;
			}
			if ($line =~ /(ENS[\S]+)[\s]+([\S\s]+)/g) {
				my $transcript = $1;
				my $value_line = $2;
				my @values = split('\t', $value_line);
				$return_expectancies{$chromosome}{'transcripts'}{$transcript} = \@values;
			}
		}
		close(RP);
		
		my %total_value;
		my %total_counts;
		my @indices = @{$return_expectancies{$chromosome}{'indices'}};
		my $transcript_hashref = $return_expectancies{$chromosome}{'transcripts'};
		foreach my $transcript (keys %$transcript_hashref) {
			my @values = @{$return_expectancies{$chromosome}{'transcripts'}{$transcript}};
			for (my $i = 0; $i < (scalar(@values) - 1); $i++) { 
				if (!defined($total_value{$indices[$i]})) { $total_value{$indices[$i]} = 0; }
				if (!defined($total_counts{$indices[$i]})) { $total_counts{$indices[$i]} = 0; }
				if ($values[$i] ne '.') {
					$total_value{$indices[$i]} += $values[$i];
					$total_counts{$indices[$i]} += 1;
				}
			}
		}
		foreach my $index (keys %total_value) {
			$total_value{$index} = $total_value{$index} / $total_counts{$index};
		}
		$return_expectancies{$chromosome}{'all'} = \%total_value;
		
	}
	return \%return_expectancies;
}

sub _process_repeat_based_expectancy_file {
	my $self = shift;
	my $expectancy_files_arrayref = shift;
	my @expectancy_files = @$expectancy_files_arrayref;

	my %return_expectancies;
	foreach my $expectancy_file (@expectancy_files) {
		my ($chromosome) = $expectancy_file =~ m/[\S\s]+\.([\S]+)\.expect$/;
		print STDERR "Opening $expectancy_file for chr: $chromosome\n";
		open(RP, $expectancy_file) || $self->throw("Cannot open expectancy file " . $expectancy_file); 
		while (my $line = <RP>) {
			chomp($line);
			if ($line =~ /##TRANSCRIPT[\s]+([\S\s]+)/g) {
				my $index_line = $1;
				my @indices = split('\t', $index_line);
				$return_expectancies{$chromosome}{'indices'} = \@indices;
			}
			if ($line =~ /(ENS[\S]+)[\s]+([\S\s]+)/g) {
				my $transcript = $1;
				my $value_line = $2;
				my @values = split('\t', $value_line);
			
				my @in_repeat_values =();
				my @long_event_values =();
				my @short_event_values =();
				my @length_values =();
				foreach my $value (@values) {
					my $in_repeat_value = ".";
					my $long_event_value = ".";
					my $short_event_value = ".";
					my $length_value = ".";
					my @valuenuggets = split('##', $value);
					foreach my $valuenugget (@valuenuggets) {
						if (defined ($valuenugget)) {
							my @datanuggets = split(":", $valuenugget);
							if (scalar(@datanuggets) > 0) {
								if ($datanuggets[0] eq "INREPEAT") {
									$in_repeat_value = $datanuggets[1];
								} elsif ($datanuggets[0] eq "SHORTEVENTS") {
									$short_event_value = $datanuggets[1];
								} elsif ($datanuggets[0] eq "LONGEVENTS") {
									$long_event_value = $datanuggets[1];
								} elsif ($datanuggets[0] eq "LENGTH") {
									$length_value= $datanuggets[1];	
								}    
							}
						}
					}
					push @in_repeat_values, $in_repeat_value;
					push @long_event_values, $long_event_value;
					push @short_event_values, $short_event_value;
					push @length_values, $length_value;
				}
				$return_expectancies{$chromosome}{'transcripts'}{$transcript}{'inrepeat'} = \@in_repeat_values;
				$return_expectancies{$chromosome}{'transcripts'}{$transcript}{'longevent'} = \@long_event_values;
				$return_expectancies{$chromosome}{'transcripts'}{$transcript}{'shortevent'} = \@short_event_values;
				$return_expectancies{$chromosome}{'transcripts'}{$transcript}{'length'} = \@length_values;
			}
		}
		close(RP);
		
		my %total_value;
		my %total_counts;
		my @indices = @{$return_expectancies{$chromosome}{'indices'}};
		my $transcript_hashref = $return_expectancies{$chromosome}{'transcripts'};
		foreach my $transcript (keys %$transcript_hashref) {
			my $analysis_types_hashref = $return_expectancies{$chromosome}{'transcripts'}{$transcript};
			foreach my $analysis_type (keys %$analysis_types_hashref) {
				my @values = @{$return_expectancies{$chromosome}{'transcripts'}{$transcript}{$analysis_type}};
				for (my $i = 0; $i < (scalar(@values) - 1); $i++) { 
					if (!defined($total_value{$analysis_type}{$indices[$i]})) { $total_value{$analysis_type}{$indices[$i]} = 0; }
					if (!defined($total_counts{$analysis_type}{$indices[$i]})) { $total_counts{$analysis_type}{$indices[$i]} = 0; }
					if ($values[$i] ne '.') {
						$total_value{$analysis_type}{$indices[$i]} += $values[$i];
						$total_counts{$analysis_type}{$indices[$i]} += 1;
					}
				}
			}
		}
		foreach my $analysis_type (keys %total_value) {
			my $values_hashref = $total_value{$analysis_type};
			foreach my $index (keys %$values_hashref) {
				$total_value{$analysis_type}{$index} = $total_value{$analysis_type}{$index} / $total_counts{$analysis_type}{$index};
			}
		}
		$return_expectancies{$chromosome}{'all'} = \%total_value;
		
	}
	return \%return_expectancies;	
}

