package Chum::Utils::Kaks;

use vars qw(@ISA @KAKS_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::EnsEMBL::Registry;
use Bio::Root::Root;
use Data::Dumper;

@ISA = qw( Bio::Root::Root );

BEGIN {
	@KAKS_PARAMS = qw();
	foreach my $attr (@KAKS_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
	my($class, @args) = @_;
	my $self = $class->SUPER::new(@args);
	while (@args) {
		my $attr = shift @args;
		my $value = shift @args;
		$self->$attr($value);
    }
    return $self;
}

sub AUTOLOAD {
    my $self = shift;
    my $attr = $AUTOLOAD;
    $attr =~ s/.*:://;

    $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
    $self->{$attr} = shift if @_;
    return $self->{$attr};
}

sub get_kaks_species_and_distances_for_gene_names_hashref {
	my $self = shift;
	my $gene_names_arrayref = shift;
	if (!defined($gene_names_arrayref)) { $self->throw("No gene names supplied for analysis"); }
	elsif (scalar(@$gene_names_arrayref) == 0) { $self->throw("No gene names supplied for analysis"); }
	
	my $ma = Bio::EnsEMBL::Registry->get_adaptor('compara','compara','Member');
	my $ha = Bio::EnsEMBL::Registry->get_adaptor('compara','compara','Homology');
	my $mlssa = Bio::EnsEMBL::Registry->get_adaptor('compara','compara','MethodLinkSpeciesSet');

	if (!defined($ma)) {
		die("MA not defined\n");
	}

	my %kaks_species_and_distance_array;
	foreach my $gene_name (@$gene_names_arrayref) {
		if (!defined(%kaks_species_and_distance_array->{$gene_name})) {
			print STDERR "GENE_NAME: " . $gene_name . "\n";
			my $member = $ma->fetch_by_source_stable_id('ENSEMBLGENE', $gene_name);
			if (defined($member)) {
				my $homologies = $ha->fetch_by_Member($member);
				my @scores = ();
				foreach my $homology (@$homologies) {
					my $dnds = $homology->dnds_ratio;
					my $method_link_species_set = $mlssa->fetch_by_dbID($homology->method_link_species_set_id);
					my @genomedbs = @{$method_link_species_set->species_set()};
					if (scalar(@genomedbs) == 2) {
						push @scores, {	'species1' => $genomedbs[0]->name,
										'species2' => $genomedbs[1]->name,
										'kaks' => $dnds };
					}
				}
				if (@scores) {
					%kaks_species_and_distance_array->{$gene_name} = \@scores;
				}
			}
		}
	}

	return \%kaks_species_and_distance_array;
}

1;
