package Chum::Utils::UpstreamRegionFinder;

use vars qw(@ISA @URF_PARAMS %OK_FIELD $AUTOLOAD);

use strict;
use Bio::EnsEMBL::Registry;
use Bio::Root::Root;

@ISA = qw( Bio::Root::Root );

BEGIN {
	@URF_PARAMS = qw();
	foreach my $attr (@URF_PARAMS)
                { $OK_FIELD{$attr}++; }
}

sub new {
	my($class, @args) = @_;
	my $self = $class->SUPER::new(@args);
	while (@args) {
		my $attr = shift @args;
		my $value = shift @args;
		$self->$attr($value);
    }
    return $self;
}

sub AUTOLOAD {
    my $self = shift;
    my $attr = $AUTOLOAD;
    $attr =~ s/.*:://;

    $self->throw("Unallowed parameter: $attr !") unless $OK_FIELD{$attr};
    $self->{$attr} = shift if @_;
    return $self->{$attr};
}

sub get_random_upstream_regions {
	my $self = shift;
	my $chromosome = shift;
	my $length = shift;
	my $number = shift;
	
	my $sa = Bio::EnsEMBL::Registry->get_adaptor("human", "core", "Slice");
	my $chr_slice = $sa->fetch_by_region('chromosome', $chromosome);
	my @genes = @{$chr_slice->get_all_Genes()};
	for (my $i = 0; $i < scalar(@genes); $i++) {
		if (!$genes[$i]->is_known) {
			splice(@genes, $i--, 1);	
		}
	}
	if ($number > scalar(@genes)) { $self->throw("Not enough known genes to generate specified number of random upstream regions in chr $chromosome"); }
	my @ret_regions = ();
	while (scalar(@ret_regions) < $number) {
		if (scalar(@genes) == 0) { $self->throw("Ran out of genes in chr $chromosome"); }
		my $index = rand(@genes);	
		my $gene = $genes[$index];
		splice(@genes, $index, 1);	
		my @transcripts = @{$gene->get_all_Transcripts};
		my $tindex = rand(@transcripts);
		my $transcript = $transcripts[$tindex];
		if ($transcript->strand eq "1") {
			my $region_start = $transcript->start - $length - 1;
			my $region_end = $transcript->start - 1;
			my %region = ('transcript' => $transcript,
				      'strand' => $transcript->strand,
				      'chromosome' => $transcript->seq_region_name,
				      'start' => $region_start,
				      'end' => $region_end);
			push @ret_regions, \%region; 
		} elsif ($transcript->strand eq "-1") {
			my $region_start = $transcript->end + 1;
			my $region_end = $transcript->end + $length + 1;
			my %region = (	'transcript' => $transcript,
					'strand' => $transcript->strand,
					'chromosome' => $transcript->seq_region_name,
					'start' => $region_start,
					'end' => $region_end);
			push @ret_regions, \%region;
		}  
	}
	return \@ret_regions;
}
