#!/usr/bin/perl

use strict;
use Data::Dumper;

my $directory = @ARGV[0];
if (!defined($directory)) { die "No directory specified"; }
unless (-d $directory) { die "Directory $directory does not exist"; }
my $output_directory = @ARGV[1];
if (!defined($output_directory)) { die "No output directory specified"; }
unless (-d $output_directory) { die "Output directory $output_directory does not exist"; }

my $mapping_ref = &get_ensg_mappings_hashref();
my @files = glob($directory . "/*.snpa.xml");
my @jobs = ();
foreach my $file (@files) {
	if ($file =~ /[\S]+(OREG[0-9]+).snpa.xml/g || $file =~ /[\S]+(rs[0-9]+).snpa.xml/g) {
		if (!defined($mapping_ref->{$1})) {
			warn("Ignoring: Cannot find gene id for " . $1);
		}
		my $job_string = "perl58 /home/smontgom/sNPeGEn/scripts/analysis/runAll.pl -x 1:2:3:4:5:6:7:8:11:12:13:14:15:16:17 -f $file -o $output_directory/" . $1 . ".results -g " . $mapping_ref->{$1} . ";";
		push @jobs, $job_string;
	}
}

my $jobs = 1;
my %superjobs;
my $index = 0;
foreach my $job (@jobs) {
	if (defined($superjobs{$index})) {
		$superjobs{$index} .= $job;
	} else {
		$superjobs{$index} = $job;
		#$superjobs{$index} = "export LD_ASSUME_KERNEL=2.4.1;export PERLEXE=" . $ENV{'PERLEXE'} . ";export CHUM_INSTALL_DIR=/home/smontgom/sNPeGEn;export ENSEMBL_REGISTRY=/home/smontgom/oregano-client-testgrounds/mappingAgent/ensembl-registry.config;export BLASTDB=/home/smontgom/databases/blast;export PERL5LIB=" . $ENV{'PERL5LIB'} . ";export PATH=/home/pubseq/BioSw/bin:/home/pubseq/BioSw/wuBLAST_dec_2005:/home/pubseq/BioSw/EMBOSS/3.0.0/bin:\$PATH;" . $job; 
	}
	$index++;
	if ($index == $jobs) { $index = 0; }
}

foreach my $key (keys %superjobs) {
	print %superjobs->{$key} . "\n";
}
1;

sub get_ensg_mappings_hashref {
	open (MAPPING, "/home/smontgom/sNPeGEn/scripts/coexp/oreganno.genes.mapped") || die("Cannot open mapping file");
	my %mappings;
	while (my $line = <MAPPING>) {
		chomp($line);
		if ($line =~ /ORegAnno:(OREG[0-9]{7})[\s\S]+EnsEMBL:([\S]+)[\s]*/gi)  {
			my $oregid = $1;
			my $ensid = $2;
			%mappings->{$oregid} = $ensid;
		}
	}
	close(MAPPING);
	
	open (GROUPS, "/home/smontgom/sNPeGEn/scripts/results/groups.txt") || die("Cannot open mapping file");
	while (my $line = <GROUPS>) {
        chomp($line);
        $line =~ s/://;
        my @ids = split(/[\s]+/, $line);
        foreach my $id (@ids) {
			if (@ids[0] eq "BAX") { %mappings->{$id} = "ENSG00000087088";}
			elsif (@ids[0] eq "BCL10") { %mappings->{$id} = "ENSG00000171791";}
			elsif (@ids[0] eq "BCL2") { %mappings->{$id} = "ENSG00000113916";}
			elsif (@ids[0] eq "BCL6") { %mappings->{$id} = "ENSG00000142867";}
			elsif (@ids[0] eq "P53") { %mappings->{$id} = "ENSG00000141510";}
			else { %mappings->{$id} = %mappings->{@ids[0]}; }
        }
	}
	close(GROUPS);
	#print Dumper \%mappings;
	return \%mappings;
}

