#!/usr/bin/perl

use strict;
use Data::Dumper;
use Chum::Analysis::Local::Banana;
use Chum::Result::Interpretter::BananaInterpretter;
use Chum::Utils::RSNPAnalysisXMLFileReader;

###############################################LOAD INPUT PARAMETERS
my $rsnpa_xml_file;
my $local = 0;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-f/) {
        $rsnpa_xml_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-l/) {
        $local = $ARGV[++$i];
    } else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if (!defined($rsnpa_xml_file)) { die("ERROR: Input file not defined: -f\n"); }
unless (-e $rsnpa_xml_file) { die("ERROR: Input file does not exist $rsnpa_xml_file\n"); }

###############################################ANALYZE INPUT FILE
my $rsnpa_obj = new Chum::Utils::RSNPAnalysisXMLFileReader(-file => $rsnpa_xml_file);
my $stack = $rsnpa_obj->get_sequence_stack_obj('hosa');
my $resultset;
if (!$local) {
    require Chum::Analysis::Remote::Banana;
    my $banana = new Chum::Analysis::Remote::Banana();
    my @bservices = @{$banana->get_remote_services};
    my $brandom_index = rand @bservices;
    my $bservice = $bservices[$brandom_index];
    print STDERR "Banana running at " . $bservice->getLocation . "\n";
    if (scalar(@bservices) > 0) {
	my @bparameters = @{$banana->get_input_parameters($bservice)};
	$resultset = $banana->run_variation_analysis($bservice, $stack, \@bparameters);
    } else {
	die "ERROR: Not running banana\n";
    }
} else {
	my $banana = new Chum::Analysis::Local::Banana();
	$resultset = $banana->run_variation_analysis($stack);

}

if (defined($resultset)) {
	my $binterpretter = new Chum::Result::Interpretter::BananaInterpretter('resultset' => $resultset);
	my $variation_name = $stack->get_variation_name;
	my $max_score = $binterpretter->calculate_maximum_score('bend');
	my $diff_score = $binterpretter->calculate_maximum_difference('bend');
	print "MAX SCORE: " . $max_score . "\n";
	print "DIFF SCORE: " . $diff_score . "\n";
} else {
	die "ERROR: No results generated\n";
}