#!/usr/bin/perl

use strict;
use Data::Dumper;
use Chum::Utils::AlleleFrequencyFinder;
use Chum::Utils::RSNPAnalysisXMLFileReader;

###############################################LOAD INPUT PARAMETERS
my $rsnpa_xml_file;
my $flank_length = 100;
my $local = 0;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-f/) {
        $rsnpa_xml_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-w/) {
        $flank_length = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-l/) {
        $local = $ARGV[++$i];
    } else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if (!defined($rsnpa_xml_file)) { die("ERROR: Input file not defined: -f\n"); }
unless (-e $rsnpa_xml_file) { die("ERROR: Input file does not exist $rsnpa_xml_file\n"); }

###############################################ANALYZE INPUT FILE
my $rsnpa_obj = new Chum::Utils::RSNPAnalysisXMLFileReader(-file => $rsnpa_xml_file);
my $stack_object = $rsnpa_obj->get_sequence_stack_obj('hosa');

my $aff = new Chum::Utils::AlleleFrequencyFinder();
my $daf = $aff->get_daf_from_orthologue($stack_object, "patr");
print Dumper $daf;
my $frequency_hashref = $daf->{'frequency'};
foreach my $population (keys %$frequency_hashref) {
    print "DAF:" . $population . ":" .  $daf->{'frequency'}->{$population}->{'allele'}->{'frequency'} ."\n";  
}

exit(0);