#!/usr/bin/perl

use strict;
use Bio::EnsEMBL::Registry;
use Data::Dumper;
use Chum::Utils::GeneFinder;
use Chum::Utils::Kaks;
use Chum::Utils::RSNPAnalysisXMLFileReader;

###############################################LOAD INPUT PARAMETERS
my $rsnpa_xml_file;
my $flank_length = 100;
my $local = 0;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-f/) {
        $rsnpa_xml_file = $ARGV[++$i];
	} elsif ($ARGV[$i] =~ /-w/) {
        $flank_length = $ARGV[++$i];
	} elsif ($ARGV[$i] =~ /-l/) {
        $local = $ARGV[++$i];
    } else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if (!defined($rsnpa_xml_file)) { die("ERROR: Input file not defined: -f\n"); }
unless (-e $rsnpa_xml_file) { die("ERROR: Input file does not exist $rsnpa_xml_file\n"); }

Bio::EnsEMBL::Registry->load_all();

###############################################ANALYZE INPUT FILE
my $rsnpa_obj = new Chum::Utils::RSNPAnalysisXMLFileReader(-file => $rsnpa_xml_file);
my $stack = $rsnpa_obj->get_sequence_stack_obj('hosa');

my $gene_finder = new Chum::Utils::GeneFinder();
my $kaks = new Chum::Utils::Kaks();

my $transcript_distance_hashref = $gene_finder->get_ensembl_transcript_distances_to_stack($stack);
my @gene_names = ();
if (defined $transcript_distance_hashref) {
	foreach my $key (keys %$transcript_distance_hashref) {
		print $key . " " . $transcript_distance_hashref->{$key} . "\n";
		my @ids = split(/:/, $key);
		if ($ids[0] ne "unknown") {
			push @gene_names, $ids[0];
		}
	}
}
my $kaks_results_hashref = $kaks->get_kaks_species_and_distances_for_gene_names_hashref(\@gene_names);
print Dumper $kaks_results_hashref;
exit(0);
