#!/usr/bin/perl

use strict;
use Data::Dumper;
use Chum::Analysis::Local::Motifsampler;
use Chum::Result::Interpretter::MotifsamplerInterpretter;
use Chum::Utils::RSNPAnalysisXMLFileReader;

###############################################LOAD INPUT PARAMETERS
my $rsnpa_xml_file;
my $flank_length = 10;
my $background_file;
my $local = 0;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-f/) {
        $rsnpa_xml_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-w/) {
        $flank_length = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-l/) {
        $local = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-b/) {
        $background_file = $ARGV[++$i];
    } else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if (!defined($rsnpa_xml_file)) { die("ERROR: Input file not defined: -f\n"); }
unless (-e $rsnpa_xml_file) { die("ERROR: Input file does not exist $rsnpa_xml_file\n"); }
if (!defined($background_file)) { die("ERROR: Background file not defined: -b\n"); }
unless (-e $background_file) { die("ERROR: Background file does not exist $background_file\n"); }

###############################################ANALYZE INPUT FILE
my $rsnpa_obj = new Chum::Utils::RSNPAnalysisXMLFileReader(-file => $rsnpa_xml_file);
my $stack = $rsnpa_obj->get_flanked_sequence_stack_obj($flank_length, 'hosa');

my $resultset;
if (!$local) {
    require Chum::Analysis::Remote::Motifsampler;
    my $motifsampler = new Chum::Analysis::Remote::Motifsampler();
    my @mservices = @{$motifsampler->get_remote_services};
    my $mrandom_index = rand @mservices;
    my $mservice = $mservices[$mrandom_index];
    print STDERR "Motifsampler running at " . $mservice->getLocation . "\n";
    if (scalar(@mservices) > 0) {
	my @mparameters = @{$motifsampler->get_input_parameters($mservice)};
	foreach my $mparameter (@mparameters) {
	    if ($mparameter->name eq "p") {
	        $mparameter->{'value'} = 0.95;
	    }
	}
	my $meta_input_hashref = $motifsampler->get_meta_input_hashref;
	$meta_input_hashref->{'background_file'} = $background_file;
	$resultset = $motifsampler->run_variation_analysis($mservice, $stack, \@mparameters, $meta_input_hashref);
    } else {
	die "ERROR: Not running Motifsampler\n";
    }
} else {
    my $motifsampler = new Chum::Analysis::Local::Motifsampler();
    $motifsampler->binary("/home/smontgom/motifsampler/MotifSampler");
    my %meta_input_hashref;
    $meta_input_hashref{'background_file'} = $background_file;
    $meta_input_hashref{'-p'} = "0.95";
    $meta_input_hashref{'-n'} = "4";
    $meta_input_hashref{'-r'} = "10";
    $resultset = $motifsampler->run_variation_analysis($stack, \%meta_input_hashref);
}

if(defined($resultset)) {
	my $minterpretter = new Chum::Result::Interpretter::MotifsamplerInterpretter('resultset' => $resultset);	
	my $variation_name = $stack->get_variation_name;
	my ($min, $max, $diff) = $minterpretter->calculate_maximum_difference();
	print STDERR $min . " " . $max . " " . $diff . "\n";
} else {
	die "ERROR: Could not get results\n";
}

exit(0);