#!/usr/bin/perl

use strict;
use Data::Dumper;
use Chum::Analysis::ExtData::RegulatoryPotential;
use Chum::Utils::RSNPAnalysisXMLFileReader;

###############################################LOAD INPUT PARAMETERS
my $rsnpa_xml_file;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-f/) {
        $rsnpa_xml_file = $ARGV[++$i];
	} else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if (!defined($rsnpa_xml_file)) { die("ERROR: Input file not defined: -f\n"); }
unless (-e $rsnpa_xml_file) { die("ERROR: Input file does not exist $rsnpa_xml_file\n"); }

###############################################ANALYZE INPUT FILE
my $rsnpa_obj = new Chum::Utils::RSNPAnalysisXMLFileReader(-file => $rsnpa_xml_file);
my $stack_object = $rsnpa_obj->get_sequence_stack_obj('hosa');

my ($chromosome, $start, $end, $strand) = &getHG17Mapping($rsnpa_obj->{'id'});
if (defined($chromosome) && defined($start) && defined($end) && defined($strand)) {
	my $rp_detector = new Chum::Analysis::ExtData::RegulatoryPotential('regulatory_potential_dir' => "/home/pubseq/databases/regulatory_potential");
	print "Average RP Score: " . $rp_detector->average_regulatory_potential_score($chromosome, $start, $end) . "\n";

} else {
	die("Cannot find coordinates\n");
}
exit(0);

##############################################SUBROUTINES

sub getHG17Mapping() {
	my $id = shift;
	my $mapping_directory = "/home/smontgom/sNPeGEn/allstacks/mapped/hg17/";
	my $file = $mapping_directory . $id . ".hg17.mapped";
	unless (-e $file) { die("Cannot find mapping for $file\n"); }
	open(FILE, $file) || die("Cannot open mapping file $file\n");
	my $chromosome;
	my $start;
	my $end;
	my $strand;
	while (my $line = <FILE>) {
		chomp($line);
		if ($line =~ /([\S]+)[\s]+([0-9]+)[\s]+([0-9]+)[\s]+([\S]+)/g) {
			$chromosome = $1;
			$start = $2;
			$end = $3;
			$strand = $4;
		}
	}
	close(FILE);
	return $chromosome, $start, $end, $strand;
}