#!/usr/bin/perl

use strict;
use Data::Dumper;
use Bio::EnsEMBL::Registry;
use Chum::Utils::RepeatContentFinder;
use Chum::Utils::RSNPAnalysisXMLFileReader;

###############################################LOAD INPUT PARAMETERS
my $rsnpa_xml_file;
my $flank_length = 10000;
my $local = 0;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-f/) {
        $rsnpa_xml_file = $ARGV[++$i];
	} elsif ($ARGV[$i] =~ /-w/) {
        $flank_length = $ARGV[++$i];
	} elsif ($ARGV[$i] =~ /-l/) {
        $local = $ARGV[++$i];
    } else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if (!defined($rsnpa_xml_file)) { die("ERROR: Input file not defined: -f\n"); }
unless (-e $rsnpa_xml_file) { die("ERROR: Input file does not exist $rsnpa_xml_file\n"); }

Bio::EnsEMBL::Registry->load_all();

###############################################ANALYZE INPUT FILE
my $rsnpa_obj = new Chum::Utils::RSNPAnalysisXMLFileReader(-file => $rsnpa_xml_file);
my $stack_object = $rsnpa_obj->get_sequence_stack_obj('hosa');

my $rcf = new Chum::Utils::RepeatContentFinder();
my $repeat_content = $rcf->get_repeat_content($stack_object, $flank_length);
print Dumper $repeat_content;
exit(0);
