#!/usr/bin/perl

use strict;
use Chum::Analysis::Local::MatrixScanner;
use Chum::Result::Interpretter::MatrixScannerInterpretter;
use Chum::Utils::RSNPAnalysisXMLFileReader;
require Chum::DB::Transfac;
use Data::Dumper;

###############################################LOAD INPUT PARAMETERS
my $rsnpa_xml_file;
my $flank_length = 10;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-f/) {
        $rsnpa_xml_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-w/) {
        $flank_length = $ARGV[++$i];
    } else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if (!defined($rsnpa_xml_file)) { die("ERROR: Input file not defined: -f\n"); }
unless (-e $rsnpa_xml_file) { die("ERROR: Input file does not exist $rsnpa_xml_file\n"); }

###############################################ANALYZE INPUT FILE
my $rsnpa_obj = new Chum::Utils::RSNPAnalysisXMLFileReader(-file => $rsnpa_xml_file);
my $stack = $rsnpa_obj->get_flanked_sequence_stack_obj($flank_length, 'hosa');

###############################################RUN TRANSFAC ANALYSIS
my $transfac_db = new Chum::DB::Transfac('transfac_dir' => "/home/pubseq/databases/TransFac/current/www/biobase/transfac/7.2/data");
$transfac_db->matrixset($transfac_db->_find_all_matrixset());
my $matrix_scanner = new Chum::Analysis::Local::MatrixScanner('matrix_db' => $transfac_db);
my $resultset = $matrix_scanner->run_variation_analysis($stack);
my $msi = new Chum::Result::Interpretter::MatrixScannerInterpretter('resultset' => $resultset);
my $score = $msi->interpret();
print "SCORE: " . $score . "\n";