#!/usr/bin/perl

use strict;
use Data::Dumper;
use Chum::Analysis::Local::Weeder;
use Chum::Result::Interpretter::WeederInterpretter;
use Chum::Utils::RSNPAnalysisXMLFileReader;

###############################################LOAD INPUT PARAMETERS
my $rsnpa_xml_file;
my $flank_length = 10;
my $local = 0;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-f/) {
        $rsnpa_xml_file = $ARGV[++$i];
	} elsif ($ARGV[$i] =~ /-w/) {
        $flank_length = $ARGV[++$i];
	} elsif ($ARGV[$i] =~ /-l/) {
        $local = $ARGV[++$i];
    } else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if (!defined($rsnpa_xml_file)) { die("ERROR: Input file not defined: -f\n"); }
unless (-e $rsnpa_xml_file) { die("ERROR: Input file does not exist $rsnpa_xml_file\n"); }

###############################################ANALYZE INPUT FILE
my $rsnpa_obj = new Chum::Utils::RSNPAnalysisXMLFileReader(-file => $rsnpa_xml_file);
my $stack = $rsnpa_obj->get_sequence_stack_obj('hosa');

my $resultset;
if (!$local) {
    require Chum::Analysis::Remote::Weeder;
    my $weeder = new Chum::Analysis::Remote::Weeder();
    my @wservices = @{$weeder->get_remote_services};
    my $wrandom_index = rand @wservices;
    my $wservice = $wservices[$wrandom_index];
    print STDERR "Weeder running at " . $wservice->getLocation . "\n";
    if (scalar(@wservices) > 0) {
	my @wparameters = @{$weeder->get_input_parameters($wservice)};
	foreach my $wparameter (@wparameters) {
	    if ($wparameter->name eq "t") {
		$wparameter->{'value'} = 500;
	    } elsif ($wparameter->name eq "s") {
		$wparameter->{'value'} = "true";
	    } elsif ($wparameter->name eq "m") {
		$wparameter->{'value'} = "true";
	    }
	}
	$resultset = $weeder->run_variation_analysis($wservice, $stack, \@wparameters);
    } else {
	die "ERROR: Not running weeder\n";
    }
} else {
	my $weeder = new Chum::Analysis::Local::Weeder();
	$weeder->{'binary'} = "/home/smontgom/chinook-internal-server/services/weeder/Weeder1.0.1/weederlaunchersm.pl";
	my $meta_input_hashref;
	$meta_input_hashref->{'-o'} = "HS";
	$meta_input_hashref->{'-r'} = "small";
	$meta_input_hashref->{'-s'} = "";
	$meta_input_hashref->{'-m'} = "";
	$meta_input_hashref->{'-t'} = "500";
	$resultset = $weeder->run_variation_analysis($stack, $meta_input_hashref);
}

##############################################################INTERPRET ANALYSIS RESULTS FOR VARIATION
if(defined($resultset)) {
	my $winterpretter = new Chum::Result::Interpretter::WeederInterpretter('resultset' => $resultset);	
	my $variation_name = $stack->get_variation_name;
	my ($min, $max, $diff) = $winterpretter->calculate_maximum_difference();
	
	print STDERR $min . " " . $max . " " . $diff . "\n";
} else {
	die "ERROR: Could not get results\n";
}