#!/usr/bin/perl

use strict;
use Data::Dumper;
use Chum::Analysis::Local::Zdna;
use Chum::Utils::RSNPAnalysisXMLFileReader;
use Chum::Result::Interpretter::ZdnaInterpretter;

###############################################LOAD INPUT PARAMETERS
my $rsnpa_xml_file;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-f/) {
       $rsnpa_xml_file = $ARGV[++$i];
    } else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if (!defined($rsnpa_xml_file)) { die("ERROR: Input file not defined: -f\n"); }
unless (-e $rsnpa_xml_file) { die("ERROR: Input file does not exist $rsnpa_xml_file\n"); }

###############################################ANALYZE INPUT FILE
my $rsnpa_obj = new Chum::Utils::RSNPAnalysisXMLFileReader(-file => $rsnpa_xml_file);
my $stack = $rsnpa_obj->get_sequence_stack_obj('hosa');

my $zdna = new Chum::Analysis::Local::Zdna();
my $resultset = $zdna->run_variation_analysis($stack);

if (defined($resultset)) {
    my $zinterpretter = new Chum::Result::Interpretter::ZdnaInterpretter('resultset' => $resultset);	
    my $variation_name = $stack->get_variation_name;
    my $max_score = $zinterpretter->calculate_maximum_score();
    my $diff_score = $zinterpretter->calculate_maximum_difference();
    print "MAX SCORE: " . $max_score . "\n";
    print "DIFF SCORE: " . $diff_score . "\n";
} else {
    die "ERROR: No results generated\n";
}
exit(0);