#!/usr/bin/perl

use strict;

###############################################LOAD INPUT PARAMETERS
my $directory;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-d/) {
        $directory = $ARGV[++$i];
	} else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if (!defined($directory)) { die("ERROR: Directory not defined: -d\n"); }
unless (-d $directory) { die("ERROR: Directory does not exist $directory\n"); }

################################################RUN GROUP STATISTICS
my @files = glob($directory . "/*.seqcomp");
print "FILE\tLOWER P\tMEAN P\tUPPER P\t95%CI\tMEAN\tSTD DEV\tREFERENCE VAL\tREF - MEAN\n";
foreach my $file (@files) {
	open(FILE, $file) || die("Cannot open $file");
	my $reference;
	my @testvalues;
	while (my $line = <FILE>) {
		chomp($line);
		if ($line =~ /REFERENCE[\s]+([\S]+)[\s]+[\S]+/gi) {
			$reference = $1;
		} elsif ($line =~ /RANDOM[\s]+([\S]+)[\s]+[\S]+/gi) {
			push @testvalues, $1;
		}
	}

	#############PRINT GROUP STATISTIC VALUES
	if (scalar(@testvalues) > 0) {
		my $mean = &mean(\@testvalues);
		my $stdev = &stdev(\@testvalues);
		my $confidence_interval = 1.960 * $stdev / sqrt(scalar(@testvalues)); #95% CI
		my ($lower_count, $mean_count, $upper_count) = &area_above_value(\@testvalues, $reference, $confidence_interval);
		print $file . "\t" . $lower_count . "\t" . $mean_count. "\t" . $upper_count . "\t" . $confidence_interval . "\t" . $mean . "\t";
		print $stdev . "\t" . $reference . "\t" . ($reference - $mean) . "\n";
	}
	close(FILE);
}

sub mean() {
	my $arrayref = shift;
	my $totalvalue = 0;
	foreach my $val (@$arrayref) {
		$totalvalue += $val;
	}
	return $totalvalue / scalar(@$arrayref);
}

sub variance() {
	my $arrayref = shift;
	my $mean = &mean($arrayref);
	my $totalvalue = 0;
	foreach my $val (@$arrayref) {
		$totalvalue += ($val - $mean) * ($val - $mean);
	}
	return $totalvalue / scalar(@$arrayref);
}

sub stdev() {
	my $arrayref = shift;
	return sqrt(&variance($arrayref));
}

sub area_above_value() {
	my $arrayref = shift;
	my $reference_value = shift;
	my $confidence_interval = shift;
	my $lcount = 0;
	my $mcount = 0;
	my $ucount = 0;
	foreach my $val (@$arrayref) {
		if ($reference_value >= ($val + $confidence_interval)) {
			$ucount++;
		} elsif ($reference_value >= $val) {
			$mcount++; $ucount++;
		} elsif ($reference_value >= ($val - $confidence_interval)) {
			$lcount++; $mcount++; $ucount++;
		}
	}
	my $total = scalar(@$arrayref);
	return ($lcount / $total, $mcount / $total, $ucount / $total);
}