#!/usr/bin/perl

use strict;
use Data::Dumper;

###############################################LOAD INPUT PARAMETERS
my $directory;
my $mapping_file;
my $samples = 100;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-d/) {
        $directory = $ARGV[++$i];
	} elsif ($ARGV[$i] =~ /-m/) {
        $mapping_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-s/) {
        $samples = $ARGV[++$i];
    } else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if (!defined($mapping_file)) { die("ERROR: Mapping file not defined: -m\n"); }
unless (-e $mapping_file) { die("ERROR: Mapping file does not exist $mapping_file\n"); }
if (!defined($directory)) { die("ERROR: Directory not defined: -d\n"); }
unless (-d $directory) { die("ERROR: Directory does not exist $directory\n"); }

open (MAPPING, $mapping_file ) || die ("ERROR: Cannot open mapping file");
my %ids;
my %mappings;
while (my $line = <MAPPING>) {
	chomp($line);
	if ($line =~ /ORegAnno:(OREG[0-9]{7})[\s\S]+EnsEMBL:([\S]+)[\s]*/gi)  {
		my $oregid = $1;
		my $ensid = $2;
	#	if (!defined(%ids->{$ensid})) {
			%mappings->{$oregid} = $ensid;
	#		%ids->{$ensid} = 1;
	#	}
	}
}
close(MAPPING);
my @files = glob ($directory . "/*.xml" );
my %filemap;
foreach my $file (@files) {
	if ($file =~ /(OREG[0-9]{7})/) {
		%filemap->{$1} = $file;
	}
}

###############FOREACH BUILD COMMAND
for my $key (keys %mappings) {
	print "perl58 sequenceCompositionTest.pl -f " . %filemap->{$key} . " -g " . %mappings->{$key} . " -s " . $samples . " > data-coexp1/" . $key . ".seqcomp &\n";
}