#!/usr/bin/perl

use strict;
use Chum::Statistics::AnalysisResult;

use Chum::Neural::Neuron::TransfacNeuron;
use Chum::Neural::Neuron::WeederNeuron;
use Chum::Neural::Neuron::MotifsamplerNeuron;
use Chum::Neural::Neuron::BendNeuron;
use Chum::Neural::Neuron::CurveNeuron;
use Chum::Neural::Neuron::GCNeuron;
use Chum::Neural::Neuron::TemperatureNeuron;
use Chum::Neural::Neuron::MonthamNeuron;

use Data::Dumper;

########################################################START THE BRAIN

my $test_group_file;
my $positive_group_file;
my $neutral_group_file;
my $function = "r";
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-i/) {
        $test_group_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-p/) {
        $positive_group_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-n/) {
        $neutral_group_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-f/) {
        $function = $ARGV[++$i];
    } else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}

if ($function eq "t") {
	if (!defined($positive_group_file)) { die("No positives groups file identified\n"); }
	unless (-e $positive_group_file) { die("Positives groups file does not exist\n"); }
	if (!defined($neutral_group_file)) { die("No neutrals groups file identified\n"); }
	unless (-e $neutral_group_file) { die("Neutrals groups file does not exist\n"); }
} elsif ($function eq "r") {
	if (!defined($test_group_file)) { die("No test file identified\n"); }
	unless (-e $test_group_file) { die("Test file does not exist\n"); }
} else {
	die ("Function: -f must be a 't' (train) or 'r' (run)");
}

#############################################################LOAD THE NEURONS
my $tneuron = new Chum::Neural::Neuron::TransfacNeuron();
my $wneuron = new Chum::Neural::Neuron::WeederNeuron();
my $mneuron = new Chum::Neural::Neuron::MotifsamplerNeuron();
my $bneuron = new Chum::Neural::Neuron::BendNeuron();
my $cneuron = new Chum::Neural::Neuron::CurveNeuron();
my $gneuron = new Chum::Neural::Neuron::GCNeuron();
my $tmpneuron = new Chum::Neural::Neuron::TemperatureNeuron();
my %mntneurons; ##Loaded for each new type;


#############################################################RUN THE FUNCTIONS
if ($function eq "t") {  ##TRAIN
	my @positives = @{&loadFile($positive_group_file)};
	my @neutrals = @{&loadFile($neutral_group_file)};
	my $mntneuron_hashref = &loadMonthamNeurons(\@positives, \@neutrals);
	
	$tneuron->train(\@positives, \@neutrals);
	$wneuron->train(\@positives, \@neutrals);
	$mneuron->train(\@positives, \@neutrals);
	$bneuron->train(\@positives, \@neutrals);
	$cneuron->train(\@positives, \@neutrals);
	$gneuron->train(\@positives, \@neutrals);
	$tmpneuron->train(\@positives, \@neutrals);
	foreach my $type (keys %$mntneuron_hashref) {
		$mntneuron_hashref->{$type}->train(\@positives, \@neutrals);
	}
} elsif ($function eq "r") {  ##RUN
	my @tests =  @{&loadFile($test_group_file)};
	my $mntneuron_hashref = &loadMonthamNeurons(\@tests);
	foreach my $test (@tests) {
		my ($tvalue, $tscore) = $tneuron->think($test);
		my ($nvalue, $nscore) = $wneuron->think($test);
		my ($mvalue, $mscore) = $mneuron->think($test);
		my ($bvalue, $bscore) = $bneuron->think($test);
		my ($cvalue, $cscore) = $cneuron->think($test);
		my ($gvalue, $gscore) = $gneuron->think($test);
		my ($tmpvalue, $tmpscore) = $tmpneuron->think($test);
		my %mntvalues;
		foreach my $type (keys %$mntneuron_hashref) {
			%mntvalues->{$type} = $mntneuron_hashref->{$type}->think($test);
		}
	}
}

exit(0);

#############################################################SUBROUTINES
sub loadFile {
	my $file = shift;
	my @results = ();
	open(FILE, $file) || die ("Cannot open $file");
	foreach my $line (<FILE>) {
		chomp($line);
		push @results, new Chum::Statistics::AnalysisResult( 'file' => $line);
	}
	close(FILE);
	return \@results;
}

sub loadMonthamNeurons {
	my $positives_arrayref = shift;
	my $neutrals_arrayref = shift;

	my %montham_types;
	foreach my $positive (@$positives_arrayref ) {
		my $hashref = $positive->{'montham'};
		if ($hashref) {
			for my $key (keys %$hashref) {
				%montham_types->{$key} = 1;
			}
		}
	}
	if (defined($neutrals_arrayref)) {
		foreach my $neutral (@$neutrals_arrayref) {
			my $hashref = $neutral->{'montham'};
			if ($hashref) {
				for my $key (keys %$hashref) {
					%montham_types->{$key} = 1;
				}
			}
		}
	}
	my %montham_neurons;
	for my $type (keys %montham_types) {
		%montham_neurons->{$type} = new Chum::Neural::Neuron::MonthamNeuron('type' => $type);
	}
	return \%montham_neurons;
}