#!/usr/bin/perl

use strict;
use Chum::Statistics::AnalysisResult;
use Data::Dumper;

my $directory;
my $groups_file;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-d/) {
        $directory = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-g/) {
        $groups_file = $ARGV[++$i];
    } else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if (!defined($directory)) { die("No results directory identified\n"); }
unless(-d $directory) { die("Results directory does not exist\n"); }
if (!defined($groups_file)) { die("No groups file identified\n"); }
unless (-e $groups_file) { die("Groups file does not exist\n"); }

open(GROUPS, $groups_file) || die("Could not open group file\n");
my $group_id = 1;
my %GROUPS_HASH;
my @POSITIVES;
my @NEUTRALS;
foreach my $line (<GROUPS>) {
	chomp($line);
	$line =~ s/://;
	my @ids = split(/[\s]+/, $line);
	my $group_name;
	if (@ids[0] =~ /OREG[0-9]+/) { $group_name = $group_id;}
	elsif (@ids[0] =~ /rs[0-9]+/) { $group_name = $group_id;}
	else {$group_name = @ids[0];}

	for (my $i = 0; $i < scalar(@ids); $i++) {
		my $results_file = $directory . "/" . $ids[$i] . ".results";
		if (-e $results_file) {
			print @ids[$i] . " EXISTS\n";
			#my $analysis_result = new Chum::Statistics::AnalysisResult( 'file' => $results_file);
			#if ($analysis_result->{'montham'} == 0) {
			#	if ($id =~ /OREG[0-9]+/g) {
			#		%GROUPS_HASH->{$group_id} = undef;
			#		last;
			#	} else {
			#		next;
			#	}
			#}
			if (!defined(%GROUPS_HASH->{$group_name})) {
				my @array = ();
				push @array, $results_file;
				%GROUPS_HASH->{$group_name} = \@array;
			} else {
				my @array = @{%GROUPS_HASH->{$group_name}};
				push @array, $results_file;
				%GROUPS_HASH->{$group_name} = \@array;
			}
			#print STDERR $ids[$i] . "\t";
			if ($group_name =~ /[0-9]+/g) {
				if (@ids[$i] =~ /OREG[0-9]+/) {
					push @POSITIVES, $results_file;		
				} elsif (@ids[$i] =~ /rs[0-9]+/) {
					push @NEUTRALS, $results_file;
				}
			}
		}
	}
	$group_id++;
}
print Dumper @POSITIVES;

open(POSITIVES, ">all.positives") || die("Cannot open all positives output file");
foreach my $positive (@POSITIVES) {
	print POSITIVES $positive . "\n";
}
close(POSITIVES);
open(NEUTRALS, ">all.neutrals") || die("Cannot open all neutrals output file");
foreach my $neutral (@NEUTRALS) {
        print NEUTRALS $neutral . "\n";
}
close(NEUTRALS);

foreach my $group (keys %GROUPS_HASH) {
	if (!defined($group)) { next; }
	open(POS, ">" . $group . ".positives") || die ("Cannot open $group positives file");
	open(NEUT, ">" . $group . ".neutrals") || die ("Cannot open $group neutrals file");
	foreach my $file (@{%GROUPS_HASH->{$group}}) {
		if ($file =~ /[\S\s]*OREG[0-9]+[\S\s]*/g) {
               		print POS $file . "\n";
		} elsif ($file =~ /[\S\s]*rs[0-9]+[\S\s]*/g) {
              		print NEUT $file . "\n";
		}
	}
	close(POS);
	close(NEUT);
}

exit(0);
