#!/usr/bin/perl

use strict;
use Chum::Statistics::AnalysisResult;
use Data::Dumper;

my $STARSON = 0;

my $results_file;
my $columns_file;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-f/) {
        $results_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-c/) {
        $columns_file = $ARGV[++$i];
    } else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if (!defined($results_file)) { die("No input file defined"); }
unless (-e $results_file) { die("Input file does not exist"); }
if (!defined($columns_file)) { die("No output columns file defined"); }

my @results = @{&loadFile($results_file)};
my $index = 1;
my %tests = (		"transfac" => $index++,
					"weeder-diff" => $index++,
					"weeder-max" => $index++,
					"motifsampler-diff" => $index++,
					"motifsampler-max" => $index++,
					"bend" => $index++,
					"curve" => $index++,
					"gc" => $index++,
					"temperature" => $index++,
#				    "distance" => $index++,
#					"ln-distance" => $index++,
					"kaks-mouse" => $index++,
					"kaks-cow" => $index++,
					"kaks-dog" => $index++,
					"kaks-rat" => $index++,
					"kaks-chimp" => $index++,
					"repeat-density" => $index++,
					"in-repeat" => $index++,
					"repeat-short-events" => $index++,
					"repeat-long-events" => $index++,
					"maf" => $index++,
					# "maf-rare" => $index++,
					# "maf-common" => $index++,
					# "maf-unknown" => $index++,
					# "mps-patr" => $index++,
					# "mps-orcu" => $index++, 
					# "mps-gaga" => $index++,
#					"mps-oran" => $index++, 
#					"mps-ovar" => $index++, 
#					"mps-capo" => $index++, 
#					"mps-popa" => $index++, 
#					"mps-ecte" => $index++, 
#					"mps-feca" => $index++, 
#					"mps-cafa" => $index++, 
#					"mps-susc" => $index++, 
#					"mps-rhma" => $index++, 
#					"mps-dano" => $index++, 
#					"mps-mylu" => $index++, 
#					"mps-mumu" => $index++, 
#					"mps-loaf" => $index++, 
#					"mps-otga" => $index++, 
#					"mps-pacy" => $index++, 
#					"mps-rano" => $index++, 
#					"mps-popy" => $index++, 
#					"mps-modo" => $index++, 
#					"mps-bota" => $index++, 
#					"mps-soar" => $index++,
#					"mps-maeu" => $index++,
#					"mps-paan" => $index++,
					"coexpression-jaspar" => $index++,
					"in-cpg" => $index++,
#					"nhgri-dnase" => $index++,
					"noble-dnase" => $index++,
					"phastcons" => $index++,
					"regulatory-potential-x5" => $index++);
print "corner";
foreach my $test (sort { %tests->{$a} <=> %tests->{$b} } keys %tests) {
	print "\t" . $test;
}
print "\n";

my $average_maf = 0;
if (defined(%tests->{'maf'})) {
	my $known = 0;
	foreach my $result (@results) {
		my $allelefreq_hashref = $result->{'allele_frequency'};
		if ($allelefreq_hashref) {
			for my $key (keys %$allelefreq_hashref) {
					if ($result->{'allele_frequency'}->{$key} < 0.5) {
						$average_maf += $result->{'allele_frequency'}->{$key};
						$known++;
						last;
					}
			}
		}
	}
	$average_maf = $average_maf / $known;
}

open(RES, ">" . $columns_file) || die ("Cannot open $columns_file file");
print RES "corner\trsnp\n";
foreach my $result (@results) {
	#print STDERR $result->{'transition-transversion'} . "\n";
	if ($result->{'montham'} && $result->{'distance'}) ## && $result->{'transition-transversion'} eq "TRANSITION") 
	{
		if ($result->{'id'} =~ /OREG[0-9]+/) {
			print RES $result->{'id'} . "\t1\n";
		} elsif ($result->{'id'} =~ /rs[0-9]+/) {
			print RES $result->{'id'} . "\t0\n";
		}

		print $result->{'id'} . "\t";
		if (defined(%tests->{'transfac'})) { print $result->{'transfac'}->{'score'} . "\t"; }
		if (defined(%tests->{'weeder-diff'})) { print $result->{'weeder'}->{'score'} . "\t"; }
		if (defined(%tests->{'weeder-max'})) { print $result->{'weeder'}->{'max_score'} . "\t"; }
		if (defined(%tests->{'motifsampler-diff'})) { 
			if ($result->{'motifsampler'}->{'score'} == 0) { 
				if ($STARSON) { 
					print "*\t";
				} else {
					print $result->{'motifsampler'}->{'score'} . "\t"; 
				}
			} else {
				print $result->{'motifsampler'}->{'score'} . "\t"; 
			}
		}
		if (defined(%tests->{'motifsampler-max'})) { 
			if ($result->{'motifsampler'}->{'max_score'} == 0) { 
				if ($STARSON) { 
					print "*\t";
				} else {
					print $result->{'motifsampler'}->{'max_score'} . "\t"; 
				}
			} else {
				print $result->{'motifsampler'}->{'max_score'} . "\t"; 
			}
		}
		if (defined(%tests->{'bend'})) { print $result->{'bend'}->{'score'} . "\t"; }
		if (defined(%tests->{'curve'})) { print $result->{'curve'}->{'score'} . "\t"; }
		if (defined(%tests->{'gc'})) { print $result->{'gc'}->{'score'} . "\t"; }
		if (defined(%tests->{'temperature'})) { print $result->{'temperature'}->{'score'} . "\t"; }
		
		###DISTANCE
		my $closest_gene;
		my $closest_distance;
		my $closest_ln_distance;
		my $distance_hashref = $result->{'distance'};
		for my $key (keys %$distance_hashref) {
			if (!defined($closest_gene)) {
				$closest_gene = $result->{'distance'}->{$key}->{'gene'};
				$closest_distance = $result->{'distance'}->{$key}->{'distance'};
				$closest_ln_distance = $result->{'distance'}->{$key}->{'ln_distance'};
			} elsif ($result->{'distance'}->{$key}->{'distance'} < $closest_distance) {
				$closest_gene = $result->{'distance'}->{$key}->{'gene'};
				$closest_distance = $result->{'distance'}->{$key}->{'distance'};
				$closest_ln_distance = $result->{'distance'}->{$key}->{'ln_distance'};
			}
		}
		if (defined(%tests->{'distance'})) { print $closest_distance . "\t"; }
		if (defined(%tests->{'ln-distance'})) { print $closest_ln_distance . "\t"; }

		###KAKS
		my %resultskaks;
		my $kaks_hashref1 = $result->{'kaks'}->{$closest_gene};
		for my $key1 (keys %$kaks_hashref1) {
			my $kaks_hashref2 = $result->{'kaks'}->{$closest_gene}->{$key1};
			for my $key2 (keys %$kaks_hashref2) {
				if (defined($result->{'kaks'}->{$closest_gene}->{$key1}->{$key2})) {
					if ($key1 eq "Mus musculus" || $key2 eq "Mus musculus") {
						%resultskaks->{"mumu"} = $result->{'kaks'}->{$closest_gene}->{$key1}->{$key2};
					} elsif ($key1 eq "Bos taurus" || $key2 eq "Bos taurus") {
						%resultskaks->{"bota"} = $result->{'kaks'}->{$closest_gene}->{$key1}->{$key2};
					} elsif ($key1 eq "Canis familiaris" || $key2 eq "Canis familiaris") {
						%resultskaks->{"cafa"} = $result->{'kaks'}->{$closest_gene}->{$key1}->{$key2};
					} elsif ($key1 eq "Rattus norvegicus" || $key2 eq "Rattus norvegicus") {
						%resultskaks->{"rano"} = $result->{'kaks'}->{$closest_gene}->{$key1}->{$key2};
					} elsif ($key1 eq "Pan troglodytes" || $key2 eq "Pan troglodytes") {
						%resultskaks->{"patr"} = $result->{'kaks'}->{$closest_gene}->{$key1}->{$key2};
					} else {
						die("Should add test for $key1 $key2 in KAKS\n");
					}
				}
			}
		}
		if (!defined(%resultskaks->{"mumu"})) { if ($STARSON) { %resultskaks->{"mumu"} = "*"; } else {%resultskaks->{"mumu"} = 0;} }
		if (!defined(%resultskaks->{"bota"})) { if ($STARSON) { %resultskaks->{"bota"} = "*"; } else {%resultskaks->{"bota"} = 0;} }
		if (!defined(%resultskaks->{"cafa"})) { if ($STARSON) { %resultskaks->{"cafa"} = "*"; } else {%resultskaks->{"cafa"} = 0;} }
		if (!defined(%resultskaks->{"rano"})) { if ($STARSON) { %resultskaks->{"rano"} = "*"; } else {%resultskaks->{"rano"} = 0;} }
		if (!defined(%resultskaks->{"patr"})) { if ($STARSON) { %resultskaks->{"patr"} = "*"; } else {%resultskaks->{"patr"} = 0;} }
		if (defined(%tests->{'kaks-mouse'})) { print %resultskaks->{"mumu"} . "\t"; }
		if (defined(%tests->{'kaks-cow'})) { print %resultskaks->{"bota"} . "\t"; }
		if (defined(%tests->{'kaks-dog'})) { print %resultskaks->{"cafa"} . "\t"; }
		if (defined(%tests->{'kaks-rat'})) { print %resultskaks->{"rano"} . "\t"; }
		if (defined(%tests->{'kaks-chimp'})) { print %resultskaks->{"patr"} . "\t"; }

		###REPEAT DENSITY
		my $repeat_density = 0;
		if ($result->{'repeat'} != 0) {
			$repeat_density = $result->{'repeat'}->{'repeat_length'} / $result->{'repeat'}->{'search_length'};
		}
		if (defined(%tests->{'repeat-density'})) { print $repeat_density . "\t"; }
		if (defined(%tests->{'in-repeat'})) { print $result->{'repeat'}->{'in_repeat'} . "\t"; }
		if (defined(%tests->{'repeat-short-events'})) { print $result->{'repeat'}->{'short_events'} . "\t"; }
		if (defined(%tests->{'repeat-long-events'})) { print $result->{'repeat'}->{'long_events'} . "\t"; }

		###MAF
		if (defined(%tests->{'maf'})) {
			my $allelefreq_hashref = $result->{'allele_frequency'};
			if ($allelefreq_hashref) {
				my $printed = 0;
				for my $key (keys %$allelefreq_hashref) {
					#	if ($result->{'allele_frequency'}->{$key} < 0.05) {
					#		if (defined(%tests->{'maf-rare'})) { print "1\t"; }
					#		if (defined(%tests->{'maf-common'})) { print "0\t"; }
					#		if (defined(%tests->{'maf-unknown'})) { print "0\t"; }
					#		last;
					#	} elsif ($result->{'allele_frequency'}->{$key} < 0.5) {
					#		if (defined(%tests->{'maf-rare'})) { print "0\t"; }
					#		if (defined(%tests->{'maf-common'})) { print "1\t"; }
					#		if (defined(%tests->{'maf-unknown'})) { print "0\t"; }
					#		last;
					#	}
					if ($result->{'allele_frequency'}->{$key} < 0.5) {
							if (defined(%tests->{'maf'})) { print ($average_maf - $result->{'allele_frequency'}->{$key}); print "\t"; $printed = 1; }
					}
				}
				unless ($printed) {
					if (defined(%tests->{'maf'})) { print $average_maf . "\t"; }
				}
			} else {
				if (defined(%tests->{'maf'})) { print $average_maf . "\t"; }
				#if (defined(%tests->{'maf-rare'})) { print "0\t"; }
				#if (defined(%tests->{'maf-common'})) { print "0\t"; }
				#if (defined(%tests->{'maf-unknown'})) { print "1\t"; }
			}
		}

		###MONTHAM OUTPUT
		my $montham_hashref = $result->{'montham'};
		my %montham;
		if ($montham_hashref) {
			for my $key (keys %$montham_hashref) {
				if ($key eq "patr") { %montham->{'patr'} = abs($result->{'montham'}->{'patr'}->{'variant_mps'} - $result->{'montham'}->{'patr'}->{'reference_mps'}); }
				elsif ($key eq "orcu") { %montham->{'orcu'} = abs($result->{'montham'}->{'orcu'}->{'variant_mps'} - $result->{'montham'}->{'orcu'}->{'reference_mps'}); }
				elsif ($key eq "gaga") { %montham->{'gaga'} = abs($result->{'montham'}->{'gaga'}->{'variant_mps'} - $result->{'montham'}->{'gaga'}->{'reference_mps'}); }
				elsif ($key eq "oran") { %montham->{'oran'} = abs($result->{'montham'}->{'oran'}->{'variant_mps'} - $result->{'montham'}->{'oran'}->{'reference_mps'}); }
				elsif ($key eq "ovar") { %montham->{'ovar'} = abs($result->{'montham'}->{'ovar'}->{'variant_mps'} - $result->{'montham'}->{'ovar'}->{'reference_mps'}); }
				elsif ($key eq "capo") { %montham->{'capo'} = abs($result->{'montham'}->{'capo'}->{'variant_mps'} - $result->{'montham'}->{'capo'}->{'reference_mps'}); }
				elsif ($key eq "popa") { %montham->{'popa'} = abs($result->{'montham'}->{'popa'}->{'variant_mps'} - $result->{'montham'}->{'popa'}->{'reference_mps'}); }
				elsif ($key eq "ecte") { %montham->{'ecte'} = abs($result->{'montham'}->{'ecte'}->{'variant_mps'} - $result->{'montham'}->{'ecte'}->{'reference_mps'}); }
				elsif ($key eq "feca") { %montham->{'feca'} = abs($result->{'montham'}->{'feca'}->{'variant_mps'} - $result->{'montham'}->{'feca'}->{'reference_mps'}); }
				elsif ($key eq "cafa") { %montham->{'cafa'} = abs($result->{'montham'}->{'cafa'}->{'variant_mps'} - $result->{'montham'}->{'cafa'}->{'reference_mps'}); }
				elsif ($key eq "susc") { %montham->{'susc'} = abs($result->{'montham'}->{'susc'}->{'variant_mps'} - $result->{'montham'}->{'susc'}->{'reference_mps'}); }
				elsif ($key eq "rhma") { %montham->{'rhma'} = abs($result->{'montham'}->{'rhma'}->{'variant_mps'} - $result->{'montham'}->{'rhma'}->{'reference_mps'}); }
				elsif ($key eq "dano") { %montham->{'dano'} = abs($result->{'montham'}->{'dano'}->{'variant_mps'} - $result->{'montham'}->{'dano'}->{'reference_mps'}); }
				elsif ($key eq "mylu") { %montham->{'mylu'} = abs($result->{'montham'}->{'mylu'}->{'variant_mps'} - $result->{'montham'}->{'mylu'}->{'reference_mps'}); }
				elsif ($key eq "mumu") { %montham->{'mumu'} = abs($result->{'montham'}->{'mumu'}->{'variant_mps'} - $result->{'montham'}->{'mumu'}->{'reference_mps'}); }
				elsif ($key eq "loaf") { %montham->{'loaf'} = abs($result->{'montham'}->{'loaf'}->{'variant_mps'} - $result->{'montham'}->{'loaf'}->{'reference_mps'}); }
				elsif ($key eq "otga") { %montham->{'otga'} = abs($result->{'montham'}->{'otga'}->{'variant_mps'} - $result->{'montham'}->{'otga'}->{'reference_mps'}); }
				elsif ($key eq "pacy") { %montham->{'pacy'} = abs($result->{'montham'}->{'pacy'}->{'variant_mps'} - $result->{'montham'}->{'pacy'}->{'reference_mps'}); }
				elsif ($key eq "rano") { %montham->{'rano'} = abs($result->{'montham'}->{'rano'}->{'variant_mps'} - $result->{'montham'}->{'rano'}->{'reference_mps'}); }
				elsif ($key eq "popy") { %montham->{'popy'} = abs($result->{'montham'}->{'popy'}->{'variant_mps'} - $result->{'montham'}->{'popy'}->{'reference_mps'}); }
				elsif ($key eq "modo") { %montham->{'modo'} = abs($result->{'montham'}->{'modo'}->{'variant_mps'} - $result->{'montham'}->{'modo'}->{'reference_mps'}); }
				elsif ($key eq "bota") { %montham->{'bota'} = abs($result->{'montham'}->{'bota'}->{'variant_mps'} - $result->{'montham'}->{'bota'}->{'reference_mps'}); }
				elsif ($key eq "soar") { %montham->{'soar'} = abs($result->{'montham'}->{'soar'}->{'variant_mps'} - $result->{'montham'}->{'soar'}->{'reference_mps'}); }
				elsif ($key eq "maeu") { %montham->{'maeu'} = abs($result->{'montham'}->{'maeu'}->{'variant_mps'} - $result->{'montham'}->{'maeu'}->{'reference_mps'}); }
				elsif ($key eq "paan") { %montham->{'paan'} = abs($result->{'montham'}->{'paan'}->{'variant_mps'} - $result->{'montham'}->{'paan'}->{'reference_mps'}); }
				else {
					die("Should implement $key for montham\n");
				}
			}
		}

		if (defined(%tests->{'mps-patr'})) { if (defined(%montham->{'patr'})) { print %montham->{'patr'} . "\t"; } else { if ($STARSON) { print "*\t"; } else { print "0\t"; } } }
		if (defined(%tests->{'mps-orcu'})) { if (defined(%montham->{'orcu'})) { print %montham->{'orcu'} . "\t"; } else { if ($STARSON) { print "*\t"; } else { print "0\t"; } } }
		if (defined(%tests->{'mps-gaga'})) { if (defined(%montham->{'gaga'})) { print %montham->{'gaga'} . "\t"; } else { if ($STARSON) { print "*\t"; } else { print "0\t"; } } }
		if (defined(%tests->{'mps-oran'})) { if (defined(%montham->{'oran'})) { print %montham->{'oran'} . "\t"; } else { if ($STARSON) { print "*\t"; } else { print "0\t"; } } }
		if (defined(%tests->{'mps-ovar'})) { if (defined(%montham->{'ovar'})) { print %montham->{'ovar'} . "\t"; } else { if ($STARSON) { print "*\t"; } else { print "0\t"; } } }
		if (defined(%tests->{'mps-capo'})) { if (defined(%montham->{'capo'})) { print %montham->{'capo'} . "\t"; } else { if ($STARSON) { print "*\t"; } else { print "0\t"; } } }
		if (defined(%tests->{'mps-popa'})) { if (defined(%montham->{'popa'})) { print %montham->{'popa'} . "\t"; } else { if ($STARSON) { print "*\t"; } else { print "0\t"; } } }
		if (defined(%tests->{'mps-ecte'})) { if (defined(%montham->{'ecte'})) { print %montham->{'ecte'} . "\t"; } else { if ($STARSON) { print "*\t"; } else { print "0\t"; } } }
		if (defined(%tests->{'mps-feca'})) { if (defined(%montham->{'feca'})) { print %montham->{'feca'} . "\t"; } else { if ($STARSON) { print "*\t"; } else { print "0\t"; } } }
		if (defined(%tests->{'mps-cafa'})) { if (defined(%montham->{'cafa'})) { print %montham->{'cafa'} . "\t"; } else { if ($STARSON) { print "*\t"; } else { print "0\t"; } } }
		if (defined(%tests->{'mps-susc'})) { if (defined(%montham->{'susc'})) { print %montham->{'susc'} . "\t"; } else { if ($STARSON) { print "*\t"; } else { print "0\t"; } } }
		if (defined(%tests->{'mps-rhma'})) { if (defined(%montham->{'rhma'})) { print %montham->{'rhma'} . "\t"; } else { if ($STARSON) { print "*\t"; } else { print "0\t"; } } }
		if (defined(%tests->{'mps-dano'})) { if (defined(%montham->{'dano'})) { print %montham->{'dano'} . "\t"; } else { if ($STARSON) { print "*\t"; } else { print "0\t"; } } }
		if (defined(%tests->{'mps-mylu'})) { if (defined(%montham->{'mylu'})) { print %montham->{'mylu'} . "\t"; } else { if ($STARSON) { print "*\t"; } else { print "0\t"; } } }
		if (defined(%tests->{'mps-mumu'})) { if (defined(%montham->{'mumu'})) { print %montham->{'mumu'} . "\t"; } else { if ($STARSON) { print "*\t"; } else { print "0\t"; } } }
		if (defined(%tests->{'mps-loaf'})) { if (defined(%montham->{'loaf'})) { print %montham->{'loaf'} . "\t"; } else { if ($STARSON) { print "*\t"; } else { print "0\t"; } } }
		if (defined(%tests->{'mps-otga'})) { if (defined(%montham->{'otga'})) { print %montham->{'otga'} . "\t"; } else { if ($STARSON) { print "*\t"; } else { print "0\t"; } } }
		if (defined(%tests->{'mps-pacy'})) { if (defined(%montham->{'pacy'})) { print %montham->{'pacy'} . "\t"; } else { if ($STARSON) { print "*\t"; } else { print "0\t"; } } }
		if (defined(%tests->{'mps-rano'})) { if (defined(%montham->{'rano'})) { print %montham->{'rano'} . "\t"; } else { if ($STARSON) { print "*\t"; } else { print "0\t"; } } }
		if (defined(%tests->{'mps-popy'})) { if (defined(%montham->{'popy'})) { print %montham->{'popy'} . "\t"; } else { if ($STARSON) { print "*\t"; } else { print "0\t"; } } }
		if (defined(%tests->{'mps-modo'})) { if (defined(%montham->{'modo'})) { print %montham->{'modo'} . "\t"; } else { if ($STARSON) { print "*\t"; } else { print "0\t"; } } }
		if (defined(%tests->{'mps-bota'})) { if (defined(%montham->{'bota'})) { print %montham->{'bota'} . "\t"; } else { if ($STARSON) { print "*\t"; } else { print "0\t"; } } }
		if (defined(%tests->{'mps-soar'})) { if (defined(%montham->{'soar'})) { print %montham->{'soar'} . "\t"; } else { if ($STARSON) { print "*\t"; } else { print "0\t"; } } }
		if (defined(%tests->{'mps-maeu'})) { if (defined(%montham->{'maeu'})) { print %montham->{'maeu'} . "\t"; } else { if ($STARSON) { print "*\t"; } else { print "0\t"; } } }
		if (defined(%tests->{'mps-paan'})) { if (defined(%montham->{'paan'})) { print %montham->{'paan'} . "\t"; } else { if ($STARSON) { print "*\t"; } else { print "0\t"; } } }
	
		if (defined(%tests->{'coexpression-jaspar'})) { if ($result->{'coexpression-jaspar'}->{'score'} eq "UNDEFINED") { if ($STARSON) { print "*\t"; } else { print "0\t"; }} else { print $result->{'coexpression-jaspar'}->{'score'} . "\t"; }}
		if (defined(%tests->{'in-cpg'})) { if ($result->{'in-cpg'}->{'score'} eq "UNDEFINED") { if ($STARSON) { print "*\t"; } else { print "0\t"; }} else { print $result->{'cpg'}->{'score'} . "\t"; }}
		if (defined(%tests->{'nhgri-dnase'})) { if ($result->{'nhgri-dnase'}->{'score'} eq "UNDEFINED") { if ($STARSON) { print "*\t"; } else { print "0\t"; }} else { print $result->{'nhgri-dnase'}->{'score'} . "\t"; }}
		if (defined(%tests->{'noble-dnase'})) { if ($result->{'noble-dnase'}->{'score'} eq "UNDEFINED") { if ($STARSON) { print "*\t"; } else { print "0\t"; }} else { print $result->{'noble-dnase'}->{'score'} . "\t"; }}
		if (defined(%tests->{'phastcons'})) { if ($result->{'phastcons'}->{'score'} eq "UNDEFINED") { if ($STARSON) { print "*\t"; } else { print "0\t"; }} else { print $result->{'phastcons'}->{'score'} . "\t"; }}
		if (defined(%tests->{'regulatory-potential-x5'})) { if ($result->{'regulatory-potential-x5'}->{'score'} eq "UNDEFINED") { if ($STARSON) { print "*\t"; } else { print "0\t"; }} else { print $result->{'rp'}->{'score'} . "\n"; }}
	}
}
close (RES);

exit(0);

sub loadFile {
	my $file = shift;
	my @results = ();
	open(FILE, $file) || die ("Cannot open $file");
	foreach my $line (<FILE>) {
		chomp($line);
		push @results, new Chum::Statistics::AnalysisResult( 'file' => $line);
	}
	close(FILE);
	return \@results;
}
