#!/usr/bin/perl

use strict;
use Chum::Statistics::AnalysisResult;
use Data::Dumper;

my $STARSON = 0;

my $groups_file;
my $raw_results_directory;
my $columns_file;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-g/) {
        $groups_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-d/) {
        $raw_results_directory = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-c/) {
        $columns_file = $ARGV[++$i];
    } else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if (!defined($groups_file)) { die("No input group file defined"); }
unless (-e $groups_file) { die("Input group file does not exist"); }
if (!defined($raw_results_directory)) { die("No raw results directory defined"); }
unless (-d $raw_results_directory) { die("Raw results directory does not exist"); }
if (!defined($columns_file)) { die("No output columns file defined"); }

my $results_hashref = &loadAnalysisResults($groups_file, $raw_results_directory);
my %results = %$results_hashref;
my %averages;
for my $gene (keys %results) {
	%averages->{$gene} = &loadAverages(%results->{$gene});  ##LOAD AVERAGES FOR STATISTICAL ANALYSIS
}

###HAVE AVERAGES, RUN TESTS
my $index = 1;
my %tests = (		"transfac" => $index++,
					"weeder-diff" => $index++,
					"weeder-max" => $index++,
					"motifsampler-diff" => $index++,
					"motifsampler-max" => $index++,
					"bend" => $index++,
					"curve" => $index++,
					"gc" => $index++,
					"temperature" => $index++,
					"distance" => $index++,
				    "ln-distance" => $index++,
					# "kaks-mouse" => $index++,
					# "kaks-cow" => $index++,
					# "kaks-dog" => $index++,
					# "kaks-rat" => $index++,
					# "kaks-chimp" => $index++,
					"repeat-density" => $index++,
					"in-repeat" => $index++,
					"repeat-short-events" => $index++,
					"repeat-long-events" => $index++,
					"maf" => $index++,
#					"mps-patr" => $index++,
#					"mps-orcu" => $index++, 
#					"mps-gaga" => $index++,
#					"mps-oran" => $index++, 
#					"mps-ovar" => $index++, 
#					"mps-capo" => $index++, 
#					"mps-popa" => $index++, 
#					"mps-ecte" => $index++, 
#					"mps-feca" => $index++, 
#					"mps-cafa" => $index++, 
#					"mps-susc" => $index++, 
#					"mps-rhma" => $index++, 
#					"mps-dano" => $index++, 
#					"mps-mylu" => $index++, 
#					"mps-mumu" => $index++, 
#					"mps-loaf" => $index++, 
#					"mps-otga" => $index++, 
#					"mps-pacy" => $index++, 
#					"mps-rano" => $index++, 
#					"mps-popy" => $index++, 
#					"mps-modo" => $index++, 
#					"mps-bota" => $index++, 
#					"mps-soar" => $index++,
#					"mps-maeu" => $index++,
#					"mps-paan" => $index++,
					"coexpression-jaspar" => $index++,
					"in-cpg" => $index++,
#					"nhgri-dnase" => $index++,
					"noble-dnase" => $index++,
					"phastcons" => $index++,
					"regulatory-potential-x5" => $index++);
print "corner";
foreach my $test (sort { %tests->{$a} <=> %tests->{$b} } keys %tests) {
	print "\t" . $test;
}
print "\n";

open(RES, ">" . $columns_file) || die ("Cannot open $columns_file file");
print RES "corner\trsnp\n";
for my $gene (keys %results) {
	my $results_hashref = %results->{$gene};
	my %ar_hash = %$results_hashref;
	for my $id (keys %ar_hash) {
		my $result = %ar_hash->{$id};
		if ($result->{'montham'}) ## && $result->{'transition-transversion'} eq "TRANSITION") 
		{
			if ($result->{'id'} =~ /OREG[0-9]+/) {
				print RES $result->{'id'} . "\t1\n";
			} elsif ($result->{'id'} =~ /rs[0-9]+/) {
				print RES $result->{'id'} . "\t0\n";
			}
			print $result->{'id'} . "\t";
			if (defined(%tests->{'transfac'})) { print (%averages->{$gene}->{'transfac'} - $result->{'transfac'}->{'score'}); print "\t"; }
			if (defined(%tests->{'weeder-diff'})) {  print (%averages->{$gene}->{'weeder-diff'} - $result->{'weeder'}->{'score'}); print "\t"; }
			if (defined(%tests->{'weeder-max'})) { print (%averages->{$gene}->{'weeder-max'} - $result->{'weeder'}->{'max_score'}); print "\t"; }
			if (defined(%tests->{'motifsampler-diff'})) { print (%averages->{$gene}->{'motifsampler-diff'} - $result->{'motifsampler'}->{'score'}); print "\t"; }
			if (defined(%tests->{'motifsampler-max'})) { print (%averages->{$gene}->{'motifsampler-max'} - $result->{'motifsampler'}->{'max_score'}); print "\t"; }
			if (defined(%tests->{'bend'})) { print (%averages->{$gene}->{'bend'} - $result->{'bend'}->{'score'}); print "\t"; }
			if (defined(%tests->{'curve'})) { print (%averages->{$gene}->{'curve'} - $result->{'curve'}->{'score'}); print "\t"; }
			if (defined(%tests->{'gc'})) { print (%averages->{$gene}->{'gc'} - $result->{'gc'}->{'score'}); print "\t"; }
			if (defined(%tests->{'temperature'})) { print (%averages->{$gene}->{'temperature'} - $result->{'temperature'}->{'score'}); print "\t"; }

			###DISTANCE
			my $closest_gene;
			my $closest_distance;
			my $closest_ln_distance;
			my $distance_hashref = $result->{'distance'};
			for my $key (keys %$distance_hashref) {
				if (!defined($closest_gene)) {
					$closest_gene = $result->{'distance'}->{$key}->{'gene'};
					$closest_distance = $result->{'distance'}->{$key}->{'distance'};
					$closest_ln_distance = $result->{'distance'}->{$key}->{'ln_distance'};
				} elsif ($result->{'distance'}->{$key}->{'distance'} < $closest_distance) {
					$closest_gene = $result->{'distance'}->{$key}->{'gene'};
					$closest_distance = $result->{'distance'}->{$key}->{'distance'};
					$closest_ln_distance = $result->{'distance'}->{$key}->{'ln_distance'};
				}
			}
			if (defined(%tests->{'distance'})) { print (%averages->{$gene}->{'distance'} - $closest_distance); print "\t"; }
			if (defined(%tests->{'ln-distance'})) { print (%averages->{$gene}->{'ln-distance'} - $closest_ln_distance); print "\t"; }

			###KAKS
			my %resultskaks;
			my $kaks_hashref1 = $result->{'kaks'}->{$closest_gene};
			for my $key1 (keys %$kaks_hashref1) {
				my $kaks_hashref2 = $result->{'kaks'}->{$closest_gene}->{$key1};
				for my $key2 (keys %$kaks_hashref2) {
					if (defined($result->{'kaks'}->{$closest_gene}->{$key1}->{$key2})) {
						if ($key1 eq "Mus musculus" || $key2 eq "Mus musculus") {
							%resultskaks->{"mumu"} = $result->{'kaks'}->{$closest_gene}->{$key1}->{$key2};
						} elsif ($key1 eq "Bos taurus" || $key2 eq "Bos taurus") {
							%resultskaks->{"bota"} = $result->{'kaks'}->{$closest_gene}->{$key1}->{$key2};
						} elsif ($key1 eq "Canis familiaris" || $key2 eq "Canis familiaris") {
							%resultskaks->{"cafa"} = $result->{'kaks'}->{$closest_gene}->{$key1}->{$key2};
						} elsif ($key1 eq "Rattus norvegicus" || $key2 eq "Rattus norvegicus") {
							%resultskaks->{"rano"} = $result->{'kaks'}->{$closest_gene}->{$key1}->{$key2};
						} elsif ($key1 eq "Pan troglodytes" || $key2 eq "Pan troglodytes") {
							%resultskaks->{"patr"} = $result->{'kaks'}->{$closest_gene}->{$key1}->{$key2};
						} else {
							die("Should add test for $key1 $key2 in KAKS\n");
						}
					}
				}
			}
			if (!defined(%resultskaks->{"mumu"})) { %resultskaks->{"mumu"} = %averages->{$gene}->{'kaks-mouse'}; }
			if (!defined(%resultskaks->{"bota"})) { %resultskaks->{"bota"} = %averages->{$gene}->{'kaks-cow'} ; }
			if (!defined(%resultskaks->{"cafa"})) { %resultskaks->{"cafa"} = %averages->{$gene}->{'kaks-dog'} ; }
			if (!defined(%resultskaks->{"rano"})) { %resultskaks->{"rano"} = %averages->{$gene}->{'kaks-rat'} ; }
			if (!defined(%resultskaks->{"patr"})) { %resultskaks->{"patr"} = %averages->{$gene}->{'kaks-chimp'} ; }
			if (defined(%tests->{'kaks-mouse'})) { print (%averages->{$gene}->{'kaks-mouse'} - %resultskaks->{"mumu"}); print "\t"; }
			if (defined(%tests->{'kaks-cow'})) { print (%averages->{$gene}->{'kaks-cow'} - %resultskaks->{"bota"}); print "\t"; }
			if (defined(%tests->{'kaks-dog'})) { print (%averages->{$gene}->{'kaks-dog'} - %resultskaks->{"cafa"}); print "\t"; }
			if (defined(%tests->{'kaks-rat'})) { print (%averages->{$gene}->{'kaks-rat'} - %resultskaks->{"rano"}); print "\t"; }
			if (defined(%tests->{'kaks-chimp'})) { print (%averages->{$gene}->{'kaks-chimp'} - %resultskaks->{"patr"}); print "\t"; }


			###REPEAT DENSITY
			my $repeat_density = 0;
			if ($result->{'repeat'} != 0) {
				$repeat_density = $result->{'repeat'}->{'repeat_length'} / $result->{'repeat'}->{'search_length'};
			}
			if (defined(%tests->{'repeat-density'})) { print (%averages->{$gene}->{'repeat-density'} - $repeat_density); print "\t"; }
			if (defined(%tests->{'in-repeat'})) { print (%averages->{$gene}->{'in-repeat'} - $result->{'repeat'}->{'in_repeat'}); print "\t"; }
			if (defined(%tests->{'repeat-short-events'})) { print (%averages->{$gene}->{'repeat-short-events'} - $result->{'repeat'}->{'short_events'}); print "\t"; }
			if (defined(%tests->{'repeat-long-events'})) { print (%averages->{$gene}->{'repeat-long-events'} - $result->{'repeat'}->{'long_events'}); print "\t"; }

			###MAF
			my $allelefreq_hashref = $result->{'allele_frequency'};
			if ($allelefreq_hashref) {
				my $printed = 0;
				for my $key (keys %$allelefreq_hashref) {
						if ($result->{'allele_frequency'}->{$key} < 0.5) {
							if (defined(%tests->{'maf'})) { print (%averages->{$gene}->{'maf'} - $result->{'allele_frequency'}->{$key}); print "\t"; }
							$printed = 1;
							last;
						}
				}
				unless ($printed) {
					if (defined(%tests->{'maf'})) { if (defined(%averages->{$gene}->{'maf'})) { print %averages->{$gene}->{'maf'}; print "\t"; } else { print "0\t"; }}
				}
			} else {
				if (defined(%tests->{'maf'})) { if (defined(%averages->{$gene}->{'maf'})) { print %averages->{$gene}->{'maf'}; print "\t"; } else { print "0\t"; }}
			}

			###MONTHAM OUTPUT
			my $montham_hashref = $result->{'montham'};
			my %montham;
			if ($montham_hashref) {
				for my $key (keys %$montham_hashref) {
					if ($key eq "patr") { %montham->{'patr'} = abs($result->{'montham'}->{'patr'}->{'variant_mps'} - $result->{'montham'}->{'patr'}->{'reference_mps'}); }
					elsif ($key eq "orcu") { %montham->{'orcu'} = abs($result->{'montham'}->{'orcu'}->{'variant_mps'} - $result->{'montham'}->{'orcu'}->{'reference_mps'}); }
					elsif ($key eq "gaga") { %montham->{'gaga'} = abs($result->{'montham'}->{'gaga'}->{'variant_mps'} - $result->{'montham'}->{'gaga'}->{'reference_mps'}); }
					elsif ($key eq "oran") { %montham->{'oran'} = abs($result->{'montham'}->{'oran'}->{'variant_mps'} - $result->{'montham'}->{'oran'}->{'reference_mps'}); }
					elsif ($key eq "ovar") { %montham->{'ovar'} = abs($result->{'montham'}->{'ovar'}->{'variant_mps'} - $result->{'montham'}->{'ovar'}->{'reference_mps'}); }
					elsif ($key eq "capo") { %montham->{'capo'} = abs($result->{'montham'}->{'capo'}->{'variant_mps'} - $result->{'montham'}->{'capo'}->{'reference_mps'}); }
					elsif ($key eq "popa") { %montham->{'popa'} = abs($result->{'montham'}->{'popa'}->{'variant_mps'} - $result->{'montham'}->{'popa'}->{'reference_mps'}); }
					elsif ($key eq "ecte") { %montham->{'ecte'} = abs($result->{'montham'}->{'ecte'}->{'variant_mps'} - $result->{'montham'}->{'ecte'}->{'reference_mps'}); }
					elsif ($key eq "feca") { %montham->{'feca'} = abs($result->{'montham'}->{'feca'}->{'variant_mps'} - $result->{'montham'}->{'feca'}->{'reference_mps'}); }
					elsif ($key eq "cafa") { %montham->{'cafa'} = abs($result->{'montham'}->{'cafa'}->{'variant_mps'} - $result->{'montham'}->{'cafa'}->{'reference_mps'}); }
					elsif ($key eq "susc") { %montham->{'susc'} = abs($result->{'montham'}->{'susc'}->{'variant_mps'} - $result->{'montham'}->{'susc'}->{'reference_mps'}); }
					elsif ($key eq "rhma") { %montham->{'rhma'} = abs($result->{'montham'}->{'rhma'}->{'variant_mps'} - $result->{'montham'}->{'rhma'}->{'reference_mps'}); }
					elsif ($key eq "dano") { %montham->{'dano'} = abs($result->{'montham'}->{'dano'}->{'variant_mps'} - $result->{'montham'}->{'dano'}->{'reference_mps'}); }
					elsif ($key eq "mylu") { %montham->{'mylu'} = abs($result->{'montham'}->{'mylu'}->{'variant_mps'} - $result->{'montham'}->{'mylu'}->{'reference_mps'}); }
					elsif ($key eq "mumu") { %montham->{'mumu'} = abs($result->{'montham'}->{'mumu'}->{'variant_mps'} - $result->{'montham'}->{'mumu'}->{'reference_mps'}); }
					elsif ($key eq "loaf") { %montham->{'loaf'} = abs($result->{'montham'}->{'loaf'}->{'variant_mps'} - $result->{'montham'}->{'loaf'}->{'reference_mps'}); }
					elsif ($key eq "otga") { %montham->{'otga'} = abs($result->{'montham'}->{'otga'}->{'variant_mps'} - $result->{'montham'}->{'otga'}->{'reference_mps'}); }
					elsif ($key eq "pacy") { %montham->{'pacy'} = abs($result->{'montham'}->{'pacy'}->{'variant_mps'} - $result->{'montham'}->{'pacy'}->{'reference_mps'}); }
					elsif ($key eq "rano") { %montham->{'rano'} = abs($result->{'montham'}->{'rano'}->{'variant_mps'} - $result->{'montham'}->{'rano'}->{'reference_mps'}); }
					elsif ($key eq "popy") { %montham->{'popy'} = abs($result->{'montham'}->{'popy'}->{'variant_mps'} - $result->{'montham'}->{'popy'}->{'reference_mps'}); }
					elsif ($key eq "modo") { %montham->{'modo'} = abs($result->{'montham'}->{'modo'}->{'variant_mps'} - $result->{'montham'}->{'modo'}->{'reference_mps'}); }
					elsif ($key eq "bota") { %montham->{'bota'} = abs($result->{'montham'}->{'bota'}->{'variant_mps'} - $result->{'montham'}->{'bota'}->{'reference_mps'}); }
					elsif ($key eq "soar") { %montham->{'soar'} = abs($result->{'montham'}->{'soar'}->{'variant_mps'} - $result->{'montham'}->{'soar'}->{'reference_mps'}); }
					elsif ($key eq "maeu") { %montham->{'maeu'} = abs($result->{'montham'}->{'maeu'}->{'variant_mps'} - $result->{'montham'}->{'maeu'}->{'reference_mps'}); }
					elsif ($key eq "paan") { %montham->{'paan'} = abs($result->{'montham'}->{'paan'}->{'variant_mps'} - $result->{'montham'}->{'paan'}->{'reference_mps'}); }
					else {
						die("Should implement $key for montham\n");
					}
				}
			}
																																										# (%averages->{$gene}->{'mps-patr'} - 0) . 
			if (defined(%tests->{'mps-patr'})) { if (defined(%montham->{'patr'})) { print (%averages->{$gene}->{'mps-patr'} - %montham->{'patr'}); print "\t"; } else { if ($STARSON) { print "*\t"; } else { print %averages->{$gene}->{'mps-patr'}; print "\t"; }}}
			if (defined(%tests->{'mps-orcu'})) { if (defined(%montham->{'orcu'})) { print (%averages->{$gene}->{'mps-orcu'} - %montham->{'orcu'}); print "\t"; } else { if ($STARSON) { print "*\t"; } else { print %averages->{$gene}->{'mps-patr'}; print "\t"; }}}
			if (defined(%tests->{'mps-gaga'})) { if (defined(%montham->{'gaga'})) { print (%averages->{$gene}->{'mps-gaga'} - %montham->{'gaga'}); print "\t"; } else { if ($STARSON) { print "*\t"; } else { print %averages->{$gene}->{'mps-patr'}; print "\t"; }}}
			if (defined(%tests->{'mps-oran'})) { if (defined(%montham->{'oran'})) { print (%averages->{$gene}->{'mps-oran'} - %montham->{'oran'}); print "\t"; } else { if ($STARSON) { print "*\t"; } else { print %averages->{$gene}->{'mps-patr'}; print "\t"; }}}
			if (defined(%tests->{'mps-ovar'})) { if (defined(%montham->{'ovar'})) { print (%averages->{$gene}->{'mps-ovar'} - %montham->{'ovar'}); print "\t"; } else { if ($STARSON) { print "*\t"; } else { print %averages->{$gene}->{'mps-patr'}; print "\t"; }}}
			if (defined(%tests->{'mps-capo'})) { if (defined(%montham->{'capo'})) { print (%averages->{$gene}->{'mps-capo'} - %montham->{'capo'}); print "\t"; } else { if ($STARSON) { print "*\t"; } else { print %averages->{$gene}->{'mps-patr'}; print "\t"; }}}
			if (defined(%tests->{'mps-popa'})) { if (defined(%montham->{'popa'})) { print (%averages->{$gene}->{'mps-popa'} - %montham->{'popa'}); print "\t"; } else { if ($STARSON) { print "*\t"; } else { print %averages->{$gene}->{'mps-patr'}; print "\t"; }}}
			if (defined(%tests->{'mps-ecte'})) { if (defined(%montham->{'ecte'})) { print (%averages->{$gene}->{'mps-ecte'} - %montham->{'ecte'}); print "\t"; } else { if ($STARSON) { print "*\t"; } else { print %averages->{$gene}->{'mps-patr'}; print "\t"; }}}
			if (defined(%tests->{'mps-feca'})) { if (defined(%montham->{'feca'})) { print (%averages->{$gene}->{'mps-feca'} - %montham->{'feca'}); print "\t"; } else { if ($STARSON) { print "*\t"; } else { print %averages->{$gene}->{'mps-patr'}; print "\t"; }}}
			if (defined(%tests->{'mps-cafa'})) { if (defined(%montham->{'cafa'})) { print (%averages->{$gene}->{'mps-cafa'} - %montham->{'cafa'}); print "\t"; } else { if ($STARSON) { print "*\t"; } else { print %averages->{$gene}->{'mps-patr'}; print "\t"; }}}
			if (defined(%tests->{'mps-susc'})) { if (defined(%montham->{'susc'})) { print (%averages->{$gene}->{'mps-susc'} - %montham->{'susc'}); print "\t"; } else { if ($STARSON) { print "*\t"; } else { print %averages->{$gene}->{'mps-patr'}; print "\t"; }}}
			if (defined(%tests->{'mps-rhma'})) { if (defined(%montham->{'rhma'})) { print (%averages->{$gene}->{'mps-rhma'} - %montham->{'rhma'}); print "\t"; } else { if ($STARSON) { print "*\t"; } else { print %averages->{$gene}->{'mps-patr'}; print "\t"; }}}
			if (defined(%tests->{'mps-dano'})) { if (defined(%montham->{'dano'})) { print (%averages->{$gene}->{'mps-dano'} - %montham->{'dano'}); print "\t"; } else { if ($STARSON) { print "*\t"; } else { print %averages->{$gene}->{'mps-patr'}; print "\t"; }}}
			if (defined(%tests->{'mps-mylu'})) { if (defined(%montham->{'mylu'})) { print (%averages->{$gene}->{'mps-mylu'} - %montham->{'mylu'}); print "\t"; } else { if ($STARSON) { print "*\t"; } else { print %averages->{$gene}->{'mps-patr'}; print "\t"; }}}
			if (defined(%tests->{'mps-mumu'})) { if (defined(%montham->{'mumu'})) { print (%averages->{$gene}->{'mps-mumu'} - %montham->{'mumu'}); print "\t"; } else { if ($STARSON) { print "*\t"; } else { print %averages->{$gene}->{'mps-patr'}; print "\t"; }}}
			if (defined(%tests->{'mps-loaf'})) { if (defined(%montham->{'loaf'})) { print (%averages->{$gene}->{'mps-loaf'} - %montham->{'loaf'}); print "\t"; } else { if ($STARSON) { print "*\t"; } else { print %averages->{$gene}->{'mps-patr'}; print "\t"; }}}
			if (defined(%tests->{'mps-otga'})) { if (defined(%montham->{'otga'})) { print (%averages->{$gene}->{'mps-otga'} - %montham->{'otga'}); print "\t"; } else { if ($STARSON) { print "*\t"; } else { print %averages->{$gene}->{'mps-patr'}; print "\t"; }}}
			if (defined(%tests->{'mps-pacy'})) { if (defined(%montham->{'pacy'})) { print (%averages->{$gene}->{'mps-pacy'} - %montham->{'pacy'}); print "\t"; } else { if ($STARSON) { print "*\t"; } else { print %averages->{$gene}->{'mps-patr'}; print "\t"; }}}
			if (defined(%tests->{'mps-rano'})) { if (defined(%montham->{'rano'})) { print (%averages->{$gene}->{'mps-rano'} - %montham->{'rano'}); print "\t"; } else { if ($STARSON) { print "*\t"; } else { print %averages->{$gene}->{'mps-patr'}; print "\t"; }}}
			if (defined(%tests->{'mps-popy'})) { if (defined(%montham->{'popy'})) { print (%averages->{$gene}->{'mps-popy'} - %montham->{'popy'}); print "\t"; } else { if ($STARSON) { print "*\t"; } else { print %averages->{$gene}->{'mps-patr'}; print "\t"; }}}
			if (defined(%tests->{'mps-modo'})) { if (defined(%montham->{'modo'})) { print (%averages->{$gene}->{'mps-modo'} - %montham->{'modo'}); print "\t"; } else { if ($STARSON) { print "*\t"; } else { print %averages->{$gene}->{'mps-patr'}; print "\t"; }}}
			if (defined(%tests->{'mps-bota'})) { if (defined(%montham->{'bota'})) { print (%averages->{$gene}->{'mps-bota'} - %montham->{'bota'}); print "\t"; } else { if ($STARSON) { print "*\t"; } else { print %averages->{$gene}->{'mps-patr'}; print "\t"; }}}
			if (defined(%tests->{'mps-soar'})) { if (defined(%montham->{'soar'})) { print (%averages->{$gene}->{'mps-soar'} - %montham->{'soar'}); print "\t"; } else { if ($STARSON) { print "*\t"; } else { print %averages->{$gene}->{'mps-patr'}; print "\t"; }}}
			if (defined(%tests->{'mps-maeu'})) { if (defined(%montham->{'maeu'})) { print (%averages->{$gene}->{'mps-maeu'} - %montham->{'maeu'}); print "\t"; } else { if ($STARSON) { print "*\t"; } else { print %averages->{$gene}->{'mps-patr'}; print "\t"; }}}
			if (defined(%tests->{'mps-paan'})) { if (defined(%montham->{'paan'})) { print (%averages->{$gene}->{'mps-paan'} - %montham->{'paan'}); print "\t"; } else { if ($STARSON) { print "*\t"; } else { print %averages->{$gene}->{'mps-patr'}; print "\t"; }}}
		
			if (defined(%tests->{'coexpression-jaspar'})) { if ($result->{'coexpression-jaspar'}->{'score'} eq "UNDEFINED") { if ($STARSON) { print "*\t"; } else { print %averages->{$gene}->{'coexpression-jaspar'}; print "\t"; } } else { print (%averages->{$gene}->{'coexpression-jaspar'} - $result->{'coexpression-jaspar'}->{'score'}); print "\t"; }}
			if (defined(%tests->{'in-cpg'})) { if ($result->{'in-cpg'}->{'score'} eq "UNDEFINED") { if ($STARSON) { print "*\t"; } else { print %averages->{$gene}->{'in-cpg'}; print "\t"; } } else { print (%averages->{$gene}->{'in-cpg'} - $result->{'cpg'}->{'score'}); print "\t"; }}
			if (defined(%tests->{'nhgri-dnase'})) { if ($result->{'nhgri-dnase'}->{'score'} eq "UNDEFINED") { if ($STARSON) { print "*\t"; } else { print %averages->{$gene}->{'nhgri-dnase'}; print "\t"; } } else { print (%averages->{$gene}->{'nhgri-dnase'} - $result->{'nhgri-dnase'}->{'score'}); print "\t"; }}
			if (defined(%tests->{'noble-dnase'})) { 
				if ($result->{'noble-dnase'}->{'score'} eq "UNDEFINED") { 
					if ($STARSON) { 
						print "*\t"; 
					} else { 
						if (defined(%averages->{$gene}->{'noble-dnase'})) {
							print %averages->{$gene}->{'noble-dnase'}; print "\t"; 
						} else {
							print "0\t"; 
						}
					} 
				} else { 
					if (defined(%averages->{$gene}->{'noble-dnase'})) { 
						print (%averages->{$gene}->{'noble-dnase'} - $result->{'noble-dnase'}->{'score'}); print "\t"; 
					} else { 
						print "0\t"; 
					}
				}
			}
			if (defined(%tests->{'phastcons'})) { if ($result->{'phastcons'}->{'score'} eq "UNDEFINED") { if ($STARSON) { print "*\t"; } else { print %averages->{$gene}->{'phastcons'}; print "\t"; }; } else { print (%averages->{$gene}->{'phastcons'} - $result->{'phastcons'}->{'score'}); print "\t"; }}
			if (defined(%tests->{'regulatory-potential-x5'})) { if ($result->{'regulatory-potential-x5'}->{'score'} eq "UNDEFINED") { if ($STARSON) { print "*"; } else { print %averages->{$gene}->{'regulatory-potential-x5'}; print ""; } } else { print (%averages->{$gene}->{'regulatory-potential-x5'} - $result->{'rp'}->{'score'}); }}
			print "\n";
		}	
	}
}
close (RES);

exit(0);

#############################################################################SUBROUTINES

sub loadAverages() {
	my $results_hashref = shift;
	my %results_hash = %$results_hashref;
	my %averages = {"transfac" => 0, "weeder-diff" => 0, "weeder-max" => 0, "motifsampler-diff" => 0, "motifsampler-max" => 0,
					"bend" => 0, "curve" => 0, "gc" => 0, "temperature" => 0, "distance" => 0, "ln-distance" => 0, "kaks-mouse" => 0,
					"kaks-cow" => 0, "kaks-dog" => 0, "kaks-rat" => 0, "kaks-chimp" => 0, "repeat-density" => 0, "in-repeat" => 0,
					"repeat-short-events" => 0, "repeat-long-events" => 0, "maf-rare" => 0, "maf-common" => 0, "maf-unknown" => 0,
					"mps-patr" => 0, "mps-orcu" => 0, "mps-gaga" => 0, "mps-oran" => 0, "mps-ovar" => 0, "mps-capo" => 0, 
					"mps-popa" => 0, "mps-ecte" => 0, "mps-feca" => 0, "mps-cafa" => 0, "mps-susc" => 0, "mps-rhma" => 0, 
					"mps-dano" => 0, "mps-mylu" => 0, "mps-mumu" => 0, "mps-loaf" => 0, "mps-otga" => 0, "mps-pacy" => 0, 
					"mps-rano" => 0, "mps-popy" => 0, "mps-modo" => 0, "mps-bota" => 0, "mps-soar" => 0,"mps-maeu" => 0,
					"mps-paan" => 0, "coexpression-jaspar" => 0, "in-cpg" => 0, "nhgri-dnase" => 0, "noble-dnase" => 0, 
					"phastcons" => 0, "regulatory-potential-x5" => 0, "maf" => 0};
	my $known = 0;
	for my $id (keys %results_hash) {
		my $result = %results_hash->{$id};

		%averages->{'transfac'} += $result->{'transfac'}->{'score'};
		%averages->{'weeder-diff'} += $result->{'weeder'}->{'score'};
		%averages->{'weeder-max'} += $result->{'weeder'}->{'max_score'};
		%averages->{'motifsampler-diff'} += $result->{'motifsampler'}->{'score'};
		%averages->{'motifsampler-max'} += $result->{'motifsampler'}->{'max_score'};
		%averages->{'bend'} += $result->{'bend'}->{'score'};
		%averages->{'curve'} += $result->{'curve'}->{'score'};
		%averages->{'gc'} += $result->{'gc'}->{'score'};
		%averages->{'temperature'} += $result->{'temperature'}->{'score'};

		my $closest_gene;
		my $closest_distance;
		my $closest_ln_distance;
		my $distance_hashref = $result->{'distance'};
		for my $key (keys %$distance_hashref) {
			if (!defined($closest_gene)) {
				$closest_gene = $result->{'distance'}->{$key}->{'gene'};
				$closest_distance = $result->{'distance'}->{$key}->{'distance'};
				$closest_ln_distance = $result->{'distance'}->{$key}->{'ln_distance'};
			} elsif ($result->{'distance'}->{$key}->{'distance'} < $closest_distance) {
				$closest_gene = $result->{'distance'}->{$key}->{'gene'};
				$closest_distance = $result->{'distance'}->{$key}->{'distance'};
				$closest_ln_distance = $result->{'distance'}->{$key}->{'ln_distance'};
			}
		}
		%averages->{'distance'} += $closest_distance;
		%averages->{'ln-distance'} += $closest_ln_distance;
		
		my %resultskaks;
		my $kaks_hashref1 = $result->{'kaks'}->{$closest_gene};
		for my $key1 (keys %$kaks_hashref1) {
			my $kaks_hashref2 = $result->{'kaks'}->{$closest_gene}->{$key1};
			for my $key2 (keys %$kaks_hashref2) {
				if (defined($result->{'kaks'}->{$closest_gene}->{$key1}->{$key2})) {
					if ($key1 eq "Mus musculus" || $key2 eq "Mus musculus") {
						%resultskaks->{"mumu"} = $result->{'kaks'}->{$closest_gene}->{$key1}->{$key2};
					} elsif ($key1 eq "Bos taurus" || $key2 eq "Bos taurus") {
						%resultskaks->{"bota"} = $result->{'kaks'}->{$closest_gene}->{$key1}->{$key2};
					} elsif ($key1 eq "Canis familiaris" || $key2 eq "Canis familiaris") {
						%resultskaks->{"cafa"} = $result->{'kaks'}->{$closest_gene}->{$key1}->{$key2};
					} elsif ($key1 eq "Rattus norvegicus" || $key2 eq "Rattus norvegicus") {
						%resultskaks->{"rano"} = $result->{'kaks'}->{$closest_gene}->{$key1}->{$key2};
					} elsif ($key1 eq "Pan troglodytes" || $key2 eq "Pan troglodytes") {
						%resultskaks->{"patr"} = $result->{'kaks'}->{$closest_gene}->{$key1}->{$key2};
					} else {
						die("Should add test for $key1 $key2 in KAKS\n");
					}
				}
			}
		}
		if (!defined(%resultskaks->{"mumu"})) { %resultskaks->{"mumu"} = 0; }
		if (!defined(%resultskaks->{"bota"})) { %resultskaks->{"bota"} = 0; }
		if (!defined(%resultskaks->{"cafa"})) { %resultskaks->{"cafa"} = 0; }
		if (!defined(%resultskaks->{"rano"})) { %resultskaks->{"rano"} = 0; }
		if (!defined(%resultskaks->{"patr"})) { %resultskaks->{"patr"} = 0; }
		%averages->{'kaks-mouse'} += %resultskaks->{"mumu"};
		%averages->{'kaks-cow'} += %resultskaks->{"bota"};
		%averages->{'kaks-mouse'} += %resultskaks->{"cafa"};
		%averages->{'kaks-mouse'} += %resultskaks->{"rano"};
		%averages->{'kaks-mouse'} += %resultskaks->{"patr"};

		my $repeat_density = 0;
		if ($result->{'repeat'} != 0) {
			$repeat_density = $result->{'repeat'}->{'repeat_length'} / $result->{'repeat'}->{'search_length'};
		}
		%averages->{'repeat-density'} += $repeat_density;
		%averages->{'in-repeat'} += $result->{'repeat'}->{'in_repeat'};
		%averages->{'repeat-short-events'} += $result->{'repeat'}->{'short_events'};
		%averages->{'repeat-long-events'} += $result->{'repeat'}->{'long_events'};

		my $allelefreq_hashref = $result->{'allele_frequency'};
		if ($allelefreq_hashref) {
			for my $key (keys %$allelefreq_hashref) {
					if ($result->{'allele_frequency'}->{$key} < 0.5) {
						%averages->{'maf'} += $result->{'allele_frequency'}->{$key};
						$known++;
						last;
					}
			}
		}

		my $montham_hashref = $result->{'montham'};
		my %montham;
		if ($montham_hashref) {
			for my $key (keys %$montham_hashref) {
				if ($key eq "patr") { %montham->{'patr'} = abs($result->{'montham'}->{'patr'}->{'variant_mps'} - $result->{'montham'}->{'patr'}->{'reference_mps'}); }
				elsif ($key eq "orcu") { %montham->{'orcu'} = abs($result->{'montham'}->{'orcu'}->{'variant_mps'} - $result->{'montham'}->{'orcu'}->{'reference_mps'}); }
				elsif ($key eq "gaga") { %montham->{'gaga'} = abs($result->{'montham'}->{'gaga'}->{'variant_mps'} - $result->{'montham'}->{'gaga'}->{'reference_mps'}); }
				elsif ($key eq "oran") { %montham->{'oran'} = abs($result->{'montham'}->{'oran'}->{'variant_mps'} - $result->{'montham'}->{'oran'}->{'reference_mps'}); }
				elsif ($key eq "ovar") { %montham->{'ovar'} = abs($result->{'montham'}->{'ovar'}->{'variant_mps'} - $result->{'montham'}->{'ovar'}->{'reference_mps'}); }
				elsif ($key eq "capo") { %montham->{'capo'} = abs($result->{'montham'}->{'capo'}->{'variant_mps'} - $result->{'montham'}->{'capo'}->{'reference_mps'}); }
				elsif ($key eq "popa") { %montham->{'popa'} = abs($result->{'montham'}->{'popa'}->{'variant_mps'} - $result->{'montham'}->{'popa'}->{'reference_mps'}); }
				elsif ($key eq "ecte") { %montham->{'ecte'} = abs($result->{'montham'}->{'ecte'}->{'variant_mps'} - $result->{'montham'}->{'ecte'}->{'reference_mps'}); }
				elsif ($key eq "feca") { %montham->{'feca'} = abs($result->{'montham'}->{'feca'}->{'variant_mps'} - $result->{'montham'}->{'feca'}->{'reference_mps'}); }
				elsif ($key eq "cafa") { %montham->{'cafa'} = abs($result->{'montham'}->{'cafa'}->{'variant_mps'} - $result->{'montham'}->{'cafa'}->{'reference_mps'}); }
				elsif ($key eq "susc") { %montham->{'susc'} = abs($result->{'montham'}->{'susc'}->{'variant_mps'} - $result->{'montham'}->{'susc'}->{'reference_mps'}); }
				elsif ($key eq "rhma") { %montham->{'rhma'} = abs($result->{'montham'}->{'rhma'}->{'variant_mps'} - $result->{'montham'}->{'rhma'}->{'reference_mps'}); }
				elsif ($key eq "dano") { %montham->{'dano'} = abs($result->{'montham'}->{'dano'}->{'variant_mps'} - $result->{'montham'}->{'dano'}->{'reference_mps'}); }
				elsif ($key eq "mylu") { %montham->{'mylu'} = abs($result->{'montham'}->{'mylu'}->{'variant_mps'} - $result->{'montham'}->{'mylu'}->{'reference_mps'}); }
				elsif ($key eq "mumu") { %montham->{'mumu'} = abs($result->{'montham'}->{'mumu'}->{'variant_mps'} - $result->{'montham'}->{'mumu'}->{'reference_mps'}); }
				elsif ($key eq "loaf") { %montham->{'loaf'} = abs($result->{'montham'}->{'loaf'}->{'variant_mps'} - $result->{'montham'}->{'loaf'}->{'reference_mps'}); }
				elsif ($key eq "otga") { %montham->{'otga'} = abs($result->{'montham'}->{'otga'}->{'variant_mps'} - $result->{'montham'}->{'otga'}->{'reference_mps'}); }
				elsif ($key eq "pacy") { %montham->{'pacy'} = abs($result->{'montham'}->{'pacy'}->{'variant_mps'} - $result->{'montham'}->{'pacy'}->{'reference_mps'}); }
				elsif ($key eq "rano") { %montham->{'rano'} = abs($result->{'montham'}->{'rano'}->{'variant_mps'} - $result->{'montham'}->{'rano'}->{'reference_mps'}); }
				elsif ($key eq "popy") { %montham->{'popy'} = abs($result->{'montham'}->{'popy'}->{'variant_mps'} - $result->{'montham'}->{'popy'}->{'reference_mps'}); }
				elsif ($key eq "modo") { %montham->{'modo'} = abs($result->{'montham'}->{'modo'}->{'variant_mps'} - $result->{'montham'}->{'modo'}->{'reference_mps'}); }
				elsif ($key eq "bota") { %montham->{'bota'} = abs($result->{'montham'}->{'bota'}->{'variant_mps'} - $result->{'montham'}->{'bota'}->{'reference_mps'}); }
				elsif ($key eq "soar") { %montham->{'soar'} = abs($result->{'montham'}->{'soar'}->{'variant_mps'} - $result->{'montham'}->{'soar'}->{'reference_mps'}); }
				elsif ($key eq "maeu") { %montham->{'maeu'} = abs($result->{'montham'}->{'maeu'}->{'variant_mps'} - $result->{'montham'}->{'maeu'}->{'reference_mps'}); }
				elsif ($key eq "paan") { %montham->{'paan'} = abs($result->{'montham'}->{'paan'}->{'variant_mps'} - $result->{'montham'}->{'paan'}->{'reference_mps'}); }
				else {
					die("Should implement $key for montham\n");
				}
			}
		}

		if (defined(%montham->{'patr'})) { %averages->{'mps-patr'} += %montham->{'patr'}; }
		if (defined(%montham->{'orcu'})) { %averages->{'mps-orcu'} += %montham->{'orcu'}; }
		if (defined(%montham->{'gaga'})) { %averages->{'mps-gaga'} += %montham->{'gaga'}; }
		if (defined(%montham->{'oran'})) { %averages->{'mps-oran'} += %montham->{'oran'}; }
		if (defined(%montham->{'ovar'})) { %averages->{'mps-ovar'} += %montham->{'ovar'}; }
		if (defined(%montham->{'capo'})) { %averages->{'mps-capo'} += %montham->{'capo'}; }
		if (defined(%montham->{'popa'})) { %averages->{'mps-popa'} += %montham->{'popa'}; }
		if (defined(%montham->{'ecte'})) { %averages->{'mps-ecte'} += %montham->{'ecte'}; }
		if (defined(%montham->{'feca'})) { %averages->{'mps-feca'} += %montham->{'feca'}; }
		if (defined(%montham->{'cafa'})) { %averages->{'mps-cafa'} += %montham->{'cafa'}; }
		if (defined(%montham->{'susc'})) { %averages->{'mps-susc'} += %montham->{'susc'}; }
		if (defined(%montham->{'rhma'})) { %averages->{'mps-rhma'} += %montham->{'rhma'}; }
		if (defined(%montham->{'dano'})) { %averages->{'mps-dano'} += %montham->{'dano'}; }
		if (defined(%montham->{'mylu'})) { %averages->{'mps-mylu'} += %montham->{'mylu'}; }
		if (defined(%montham->{'mumu'})) { %averages->{'mps-mumu'} += %montham->{'mumu'}; }
		if (defined(%montham->{'loaf'})) { %averages->{'mps-loaf'} += %montham->{'loaf'}; }
		if (defined(%montham->{'otga'})) { %averages->{'mps-otga'} += %montham->{'otga'}; }
		if (defined(%montham->{'pacy'})) { %averages->{'mps-pacy'} += %montham->{'pacy'}; }
		if (defined(%montham->{'rano'})) { %averages->{'mps-rano'} += %montham->{'rano'}; }
		if (defined(%montham->{'popy'})) { %averages->{'mps-popy'} += %montham->{'popy'}; }
		if (defined(%montham->{'modo'})) { %averages->{'mps-modo'} += %montham->{'modo'}; }
		if (defined(%montham->{'bota'})) { %averages->{'mps-bota'} += %montham->{'bota'}; }
		if (defined(%montham->{'soar'})) { %averages->{'mps-soar'} += %montham->{'soar'}; }
		if (defined(%montham->{'maeu'})) { %averages->{'mps-maeu'} += %montham->{'maeu'}; }
		if (defined(%montham->{'paan'})) { %averages->{'mps-paan'} += %montham->{'paan'}; }

		if ($result->{'coexpression-jaspar'}->{'score'} ne "UNDEFINED") { %averages->{'coexpression-jaspar'} += $result->{'coexpression-jaspar'}->{'score'}; }
		if ($result->{'in-cpg'}->{'score'} ne "UNDEFINED") { %averages->{'in-cpg'} += $result->{'cpg'}->{'score'}; }
		if ($result->{'nhgri-dnase'}->{'score'} ne "UNDEFINED") { %averages->{'nhgri-dnase'} += $result->{'nhgri-dnase'}->{'score'}; }
		if ($result->{'noble-dnase'}->{'score'} ne "UNDEFINED") { %averages->{'noble-dnase'} += $result->{'noble-dnase'}->{'score'}; }
		if ($result->{'phastcons'}->{'score'} ne "UNDEFINED") { %averages->{'phastcons'} += $result->{'phastcons'}->{'score'}; }
		if ($result->{'regulatory-potential-x5'}->{'score'} ne "UNDEFINED") { %averages->{'regulatory-potential-x5'} += $result->{'rp'}->{'score'}; }
	}

	for my $key (keys %averages) {
		if ($key ne "maf") {
			%averages->{$key} = %averages->{$key} / scalar(keys(%results_hash));
		} else {
			if ($known != 0) {
				%averages->{$key} = %averages->{$key} / $known;
			} else {
				%averages->{$key} = 0;
			}
		}
	}
	return \%averages;
}

sub loadAnalysisResults() {
	my $groups_file = shift;
	my $results_directory = shift;
	my %result_hash;

	open(GROUPS, $groups_file) || die("Cannot open groups file");
	foreach my $line (<GROUPS>) {
		chomp($line);
		$line =~ s/://;
		my @ids = split(/[\s]+/, $line);
		my $geneid;
		foreach my $id (@ids) {
			eval {
				my $ar = new Chum::Statistics::AnalysisResult( 'file' => $results_directory . "/" . $id . ".results");
				my $gene = &getClosestGene($ar);
				if (!defined($geneid)) {
					$geneid = $gene;
				}
				if ($gene ne $geneid) {
					warn("$id doesnt match group: " . @ids[0]);
				} else {
					%result_hash->{$geneid}->{$id} = $ar;
				}
			};
			if (@$) {
				print STDERR "Error detected: " . @$ . "\n";
			}
		}
		
	}
	close(GROUPS);
	return \%result_hash;
}

sub getClosestGene() {
	my $result = shift;

	my $closest_gene;
	my $closest_distance;
	my $closest_ln_distance;
	my $distance_hashref = $result->{'distance'};
	for my $key (keys %$distance_hashref) {
		if (!defined($closest_gene)) {
			$closest_gene = $result->{'distance'}->{$key}->{'gene'};
			$closest_distance = $result->{'distance'}->{$key}->{'distance'};
			$closest_ln_distance = $result->{'distance'}->{$key}->{'ln_distance'};
		} elsif ($result->{'distance'}->{$key}->{'distance'} < $closest_distance) {
			$closest_gene = $result->{'distance'}->{$key}->{'gene'};
			$closest_distance = $result->{'distance'}->{$key}->{'distance'};
			$closest_ln_distance = $result->{'distance'}->{$key}->{'ln_distance'};
		}
	}
	return $closest_gene;
}