#!/usr/bin/perl

use strict;
use Chum::Statistics::AnalysisResult;

my $positives_file;
my $neutrals_file;
my $oreg_dbsnp_map_file;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-p/) {
        $positives_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-n/) {
        $neutrals_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-m/) {
        $oreg_dbsnp_map_file = $ARGV[++$i];
    } else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if (!defined($positives_file)) { die("No positives file identified\n"); }
unless(-e $positives_file) { die("Positives file does not exist\n"); }
if (!defined($neutrals_file)) { die("No neutrals file identified\n"); }
unless(-e $neutrals_file) { die("Neutrals file does not exist\n"); }
if (!defined($oreg_dbsnp_map_file)) { die("No ORegAnno-dbSNP file identified\n"); }
unless(-e $oreg_dbsnp_map_file) { die("ORegAnno-dbSNP file does not exist\n"); }

my $OREG_DBSNP_MAP = &loadOregTodbSNPMap($oreg_dbsnp_map_file);
my %positives;
open (POSITIVES, $positives_file) || die("Cannot open positives file\n");
foreach my $line (<POSITIVES>) {
    chomp($line);
    if (-e $line) {
        $positives{$line} = new Chum::Statistics::AnalysisResult( 'file' => $line);
    } else {
        die("Unidentified file $line in positives\n");   
    }
}
close(POSITIVES);

my %neutrals;
open (NEUTRALS, $neutrals_file) || die("Cannot open neutrals file\n");
foreach my $line (<NEUTRALS>) {
    chomp($line);
    if (-e $line) {
        if ($line =~ /[\S\s]*(rs[0-9]+).[\S\s]*/g) {
            my $id = $1;
            if (!defined($OREG_DBSNP_MAP->{$id})) {
                $neutrals{$line} = new Chum::Statistics::AnalysisResult( 'file' => $line);
            }
        } else {
            die("Poorly named file $line in neutrals\n");    
        }
    } else {
        die("Unidentified file $line in neutrals\n");   
    }
}
close(NEUTRALS);

#################################RUN STATISTICS ON NEUTRALS AND POSITIVES RESULTS

print "POSITIVES\n";
&printFirstFiveTestResults(\%positives);
print "NEUTRALS\n";
&printFirstFiveTestResults(\%neutrals);

################################DONE

exit(0);

###############################SUBROUTINES

sub loadOregTodbSNPMap () {
    my $map_file = shift;
    open (MAP, $map_file) || die("Cannot open oreganno to dbsnp mapping file");
    my %MAPPING;
    foreach my $line (<MAP>) {
        if ($line =~ /(OREG[0-9]{7})[\S\s]*(rs[0-9]+)[\S\s]*/g) {
            $MAPPING{$2} = $1;   
        }
    }
    close(MAP);
    return \%MAPPING;
}

sub printFirstFiveTestResults () {
    my $results_hashref = shift;
    for my $key (keys %$results_hashref) {
        print $results_hashref->{$key}->{'id'} . "\t";
        print $results_hashref->{$key}->{'transfac'}->{'score'} . "\t";
        print $results_hashref->{$key}->{'weeder'}->{'score'} . "\t";
        print $results_hashref->{$key}->{'motifsampler'}->{'score'} . "\t";
        print $results_hashref->{$key}->{'bend'}->{'score'} . "\t";
        print $results_hashref->{$key}->{'curve'}->{'score'} . "\t";
        print $results_hashref->{$key}->{'gc'}->{'score'} . "\t";
        print $results_hashref->{$key}->{'temperature'}->{'score'} . "\n";
    }
}
