#!/usr/bin/perl
use strict;

use strict;
use Chum::Statistics::AnalysisResult;
use Data::Dumper;

my $directory;
my $groups_file;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-d/) {
        $directory = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-g/) {
        $groups_file = $ARGV[++$i];
    } else {
        print STDERR "\nInvalid parameter: " . $ARGV[$i] . "\n";
        exit(1);
    }
}
if (!defined($directory)) { die("No results directory identified\n"); }
unless(-d $directory) { die("Results directory does not exist\n"); }
if (!defined($groups_file)) { die("No groups file identified\n"); }
unless (-e $groups_file) { die("Groups file does not exist\n"); }

open(GROUPS, $groups_file) || die("Could not open group file\n");
my $group_id = 1;
foreach my $line (<GROUPS>) {
    chomp($line);
    $line =~ s/://;
    my @ids = split(/[\s]+/, $line);
    foreach my $id (@ids) {
        if (-e $directory . "/" . $id . ".results") {
            print $group_id . "\t" . $id . "\t";
            my $ar = new Chum::Statistics::AnalysisResult( 'file' => $directory . "/" . $id . ".results");
            &analyzeResultContent($ar);
        } else {
            print $group_id . "\t" . $id;
        }
        print "\n";
    }
    $group_id++;
}
close(GROUPS);
exit(0);

sub analyzeResultContent() {
    my $ar = shift;    
    #A1
    if (defined($ar->{'transfac'}->{'score'}) && $ar->{'transfac'}->{'score'} ne "UNDEFINED") {
        print "OK\t";    
    } else {
        print "ERROR\t";   
    }
    
    #A2
    if (defined($ar->{'weeder'}->{'score'}) && $ar->{'weeder'}->{'score'} ne "UNDEFINED") {
	if (defined($ar->{'weeder'}->{'max_score'}) && $ar->{'weeder'}->{'max_score'} ne "UNDEFINED") {
        	print "OK\t";
	} else {
		print "ERROR\t";
	}
    } else {
        print "ERROR\t";   
    }
    
    #A3
    if (defined($ar->{'motifsampler'}->{'score'}) && $ar->{'motifsampler'}->{'score'} ne "UNDEFINED") {
	if (defined($ar->{'motifsampler'}->{'max_score'}) && $ar->{'motifsampler'}->{'max_score'} ne "UNDEFINED") {
        	print "OK\t";
	} else {
		print "ERROR\t";
	}
    } else {
        print "ERROR\t";   
    }
    
    #A4
    if (defined($ar->{'bend'}->{'score'}) && $ar->{'bend'}->{'score'} ne "UNDEFINED" &&
        defined($ar->{'curve'}->{'score'}) && $ar->{'curve'}->{'score'} ne "UNDEFINED" ) {
        print "OK\t";    
    } else {
        print "ERROR\t";   
    }
    
    #A5
    if (defined($ar->{'gc'}->{'score'}) && $ar->{'gc'}->{'score'} ne "UNDEFINED" &&
        defined($ar->{'temperature'}->{'score'}) && $ar->{'temperature'}->{'score'} ne "UNDEFINED" ) {
        print "OK\t";    
    } else {
        print "ERROR\t";   
    }
    
    #A6
    if (!defined($ar->{'distance'}) || $ar->{'distance'} == 0) {
        print "ERROR\t";   
    } else {
        print "OK\t"; 
    }
    
    #A7
    if (!defined($ar->{'kaks'}) || $ar->{'kaks'} == 0) {
        print "ERROR\t";   
    } else {
        print "OK\t"; 
    }
    
    #A8
    if (!defined($ar->{'repeat'}) || $ar->{'repeat'} == 0) {
        print "ERROR\t";   
    } else {
		print "OK\t"; 
    }
    
    #A9
    if (!defined($ar->{'allele_frequency'}) || $ar->{'allele_frequency'} == 0) {
        print "ERROR\t";   
    } else {
        print "OK\t"; 
    }
    
    #A10
    if (!defined($ar->{'daf'}) || $ar->{'daf'} == 0) {
        print "ERROR\t";
    } else {
        print "OK\t";
    }

    #A11
    if (defined($ar->{'transition-transversion'}) && $ar->{'transition-transversion'} ne "0") {
        print "OK\t";
    } else {
        print "ERROR\t";
    }

    #A12
    if (defined($ar->{'coexpression-jaspar'}->{'score'})) {
        print "OK\t";
    } else {
        print "ERROR\t";
    }

    #A13
    if (defined($ar->{'cpg'}->{'score'})) {
        print "OK\t";
    } else {
        print "ERROR\t";
    }

    #A14
    if (defined($ar->{'nhgri-dnase'}->{'score'})) {
        print "OK\t";
    } else {
        print "ERROR\t";
    }

    #A15
    if (defined($ar->{'noble-dnase'}->{'score'})) {
        print "OK\t";
    } else {
        print "ERROR\t";
    }

    #A16
    if (defined($ar->{'phastcons'}->{'score'})) {
        print "OK\t";
    } else {
        print "ERROR\t";
    }

    #A17
    if (defined($ar->{'rp'}->{'score'})) {
        print "OK\t";
    } else {
        print "ERROR\t";
    }
    
    #A18
    if (!defined($ar->{'alignment_distance'})) {
        print "ERROR\t";
    } else {
        print "OK\t";
    }
}
