#!/usr/bin/perl -w
use strict;
use XML::Twig;

##############################PROCESS INPUTS##############################
my $output_file = undef;
for (my $i = 0; $i < @ARGV; $i++) {
    if ($ARGV[$i] =~ /-h/ || $ARGV[$i] =~ /--help/) {
        print STDERR "weederToXML.pl usage\n";
        print STDERR "\t-f: specify input file\n";
        print STDERR "\tor use pipe redirect. |\n";
        exit(1);
    } elsif ($ARGV[$i] =~ /-o/) {
        $output_file = $ARGV[++$i];
    } elsif ($ARGV[$i] =~ /-f/) {
        $i++; ##Ignore parameter for now...
    } else {
        print STDERR "ERROR: Undefined option: " . $ARGV[$i] . "\n";
        exit(1);
    }
}


##############################READ IN INPUT FROM PIPE OR FROM INPUT FILE##############################
my @lines = ();
if (-p STDIN) {
    while(<STDIN>) {
        my $line = $_;
        push @lines, $line; 
    }
} else {
    my $input_file = undef;
    for (my $i = 0; $i < @ARGV; $i++) {
        if ($ARGV[$i] =~ /-f/) {
            $input_file = $ARGV[++$i];
        } 
    }
    if (!defined($input_file) || !(-f $input_file)) {
        print STDERR "ERROR: Input file is undefined or does not exist, -f: " . $input_file . "\n";
        exit(1);    
    }
    open(INPUT, $input_file) || die("Cannot open input file: " . $input_file . "\n");
    while (my $line = <INPUT>) { push @lines, $line; }
    close(INPUT);
}
if (scalar(@lines) == 0) { die("ERROR: Input is empty\n"); }


##############################TRANSFORM OUTPUT INTO XML##############################
my $root = XML::Twig::Elt->new('weeder_output');
my %scored_motifs;
my $crossed_predictions = 0;
my $new_block = 1;
my $current_motif = undef;
my $current_seqname = undef;
my $current_strand;
my $predicted_motif_tag;
my $frequency_matrix_active = 0;
my $all_frequency_matrix_tag;
my $best_frequency_matrix_tag;
foreach my $line (@lines) {
    chomp($line);
    
    ###PROCESS INITIAL PREDICTIONS
    if (!$crossed_predictions) {
        if ($line =~ /^([0-9]+)\)[\s]+([ACTGactgNn]+)[\s]+([\S]+)$/ig) {
            my $motif = $2;
            my $score = $3;
            if (defined($scored_motifs{$motif})) { die("Motif " . $motif. " has been scored twice\n"); }
            $scored_motifs{$motif} = $score;
        } elsif ($line =~ /MY[\s]+ADVICE/i) {
            $crossed_predictions = 1;  
        }
    
    ###PROCESS HITS AND PFMS FROM RESULTS ANALYZER
    } else {
        if ($new_block && !$frequency_matrix_active && $line =~ /^[\s]{1}([ATCGatcgNn]+)[\s]{1}$/g) {
            $current_motif = $1;
            $predicted_motif_tag = XML::Twig::Elt->new('predicted_motif');
            $predicted_motif_tag->set_att('seq', $current_motif);
            $predicted_motif_tag->set_att('score', $scored_motifs{$current_motif});
            $new_block = 0;
        } elsif (!$frequency_matrix_active && $line =~ /(^>|^REVCOMP[\s]+>)([\S]+)/) {
            $current_seqname = $2;
            if ($line =~ /REVCOMP/) { $current_strand = "-1"; }
            else { $current_strand = "1"; }
        } elsif (!$frequency_matrix_active && $line =~ /([ACTGNatcgn]+)[\s\S]+position[\s]+([0-9]+),[\s]+\(([\S]+)\)/i) {
            my $hit = $1;
            my $position = $2;
            my $hit_score = $3;
            my $hit_tag = XML::Twig::Elt->new('hit');
            $hit_tag->set_att('name', $current_seqname);
            $hit_tag->set_att('start', $position);
            $hit_tag->set_att('score', $hit_score);
            $hit_tag->set_att('strand', $current_strand);
            $hit_tag->set_att('sequence', $hit);
            $hit_tag->paste($predicted_motif_tag);
        } elsif ($line =~ /Frequency[\s]+Matrix/i) {
            $frequency_matrix_active = 1;
            $all_frequency_matrix_tag = XML::Twig::Elt->new('matrix');
            $all_frequency_matrix_tag->set_att('name', "All occurrences");
            $all_frequency_matrix_tag->set_att('type', "PFM");
            $best_frequency_matrix_tag = XML::Twig::Elt->new('matrix');
            $best_frequency_matrix_tag->set_att('name', "Best occurrences");
            $best_frequency_matrix_tag->set_att('type', "PFM");
        } elsif ($line =~ /([0-9]+)[\s]+([0-9]+)[\s]+([0-9]+)[\s]+([0-9]+)[\s]+([0-9]+)[\s]+([0-9]+)[\s]+([0-9]+)[\s]+([0-9]+)[\s]+([0-9]+)/g) {
            my $all_frequency_matrix_index_tag = XML::Twig::Elt->new('index');
            $all_frequency_matrix_index_tag->set_att('position', $1);
            $all_frequency_matrix_index_tag->set_att('A', $2);
            $all_frequency_matrix_index_tag->set_att('C', $3);
            $all_frequency_matrix_index_tag->set_att('G', $4);
            $all_frequency_matrix_index_tag->set_att('T', $5);
            $all_frequency_matrix_index_tag->paste($all_frequency_matrix_tag);
            
            my $best_frequency_matrix_index_tag = XML::Twig::Elt->new('index');
            $best_frequency_matrix_index_tag->set_att('position', $1);
            $best_frequency_matrix_index_tag->set_att('A', $6);
            $best_frequency_matrix_index_tag->set_att('C', $7);
            $best_frequency_matrix_index_tag->set_att('G', $8);
            $best_frequency_matrix_index_tag->set_att('T', $9);
            $best_frequency_matrix_index_tag->paste($best_frequency_matrix_tag);
            
        } elsif ($line =~ /^[=]+$/g) {
            $new_block = 1;
            $frequency_matrix_active = 0;
            if (defined($all_frequency_matrix_tag)) {
                $all_frequency_matrix_tag->paste($predicted_motif_tag);
                $all_frequency_matrix_tag = undef;
            }
            if (defined($best_frequency_matrix_tag)) {
                $best_frequency_matrix_tag->paste($predicted_motif_tag);
                $best_frequency_matrix_tag = undef;
            }
            $predicted_motif_tag->paste($root);
        }
    }
}

##############################OUTPUT RESULTS##############################
$root->set_pretty_print( 'indented');
if (defined($output_file)) {
    open(OUT, ">" , $output_file) || die("Cannot open output file");
    $root->print( \*OUT );
    close(OUT);
} else {
    $root->print;
}
exit(0);